angular.module('Flexikon', [
  'ionic'
  'templates'
  'ngIOS9UIWebViewPatch'
  'ngCordova'
  'ngCordova.plugins'
  'ngCordova.plugins.vibration'
  'ngMockE2E'
  'ionic.ion.imageCacheFactory'
  'Flexikon.controllers'
  'Flexikon.services'
  'Flexikon.directives'
  'Flexikon.translations'
  'http-auth-interceptor'
  'http-auth-interceptor-buffer'
  'pouchdb'
]).run ($ionicPlatform, $cordovaStatusbar, $http, $httpBackend, $translate, $rootScope, $templateCache, Login, LastSeen, GlobalSettings, Api, FlexikonApi, Theme, Store, InAppPurchase) ->

  ###
  # Initial config
  ###
  GlobalSettings.init()
  Login.init()
  Store.initialize()

  Theme.select(Theme.getSelected())

  if Api.getEnv() == 'development_mocked'
    $httpBackend.whenGET(/.*templates\/.*/).passThrough()
    $httpBackend.whenGET(/.*tv.doccheck.com\/.*/).passThrough()
    $httpBackend.whenPOST(/.*login.doccheck.com\/.*/).passThrough()
    $httpBackend.whenGET(/.*login.doccheck.com\/.*/).passThrough()
  else
    $httpBackend.whenGET(/.*/).passThrough()
    $httpBackend.whenPOST(/.*/).passThrough()

  ###
  if Api.getEnv().indexOf('development') >= 0 && ionic.Platform.isWebView()
    analytics.debugMode()
  ###

  $ionicPlatform.ready ->
    $rootScope.$broadcast('run-and-ready')

    $http.defaults.headers.post['Content-Type'] = false
    $http.defaults.headers.post['Allow-Control-Allow-Origin'] = '*'

    if ionic.Platform.isWebView()
      device = ionic.Platform.device()

      deviceInformation = [
        'os=' + device.platform
        'os-version=' + device.version
        'manufacturer=' + device.manufacturer
        'device=' + device.model
      ]

      $http.defaults.headers.common['X-Doccheck-App'] = 'Flexikon App/1.7.0; ' + deviceInformation.join('; ')

    # ensure app is in fullscreen and shows the status bar
    ionic.Platform.fullScreen(true, true)
    
    cordova?.plugins?.Keyboard?.disableScroll(true)
    
    # ensures that the init for the facebook tracking is triggered
    # won't ever need the login status ;)
    facebookConnectPlugin?.getLoginStatus()

    GlobalSettings.init()

    $translate.use(GlobalSettings.getExtendedLanguage())

    # immediately add .keyboad-open class on keyboardshow event (not after 300ms)
    window.addEventListener('native.keyboardshow', () ->
      document.body.classList.add('keyboard-open')
    )

    return
  return

.config ($stateProvider, $urlRouterProvider, $ionicConfigProvider, $provide, $translateProvider, $httpProvider) ->
  
  ###
  # Tab config
  ###

  $provide.decorator '$httpBackend', ($delegate) ->

    proxy = (method, url, data, callback, headers) ->

      interceptor = ->
        _this = this
        _arguments = arguments
        setTimeout(->
          callback.apply _this, _arguments
          return
        , 1)
        return

      $delegate.call this, method, url, data, interceptor, headers

    for key of $delegate
      proxy[key] = $delegate[key]
    proxy

  $ionicConfigProvider.views.swipeBackEnabled(false)
  $ionicConfigProvider.tabs.position("bottom")
  $ionicConfigProvider.tabs.style("standard")
  $ionicConfigProvider.backButton.text('').icon('fa fa-chevron-left')
  $ionicConfigProvider.backButton.previousTitleText(false)
  $ionicConfigProvider.views.maxCache(100)
  $ionicConfigProvider.views.forwardCache(true)
  $ionicConfigProvider.scrolling.jsScrolling(false)

  if ionic.Platform.isAndroid()
    # set color for status bar background while app is opened
    StatusBar?.backgroundColorByHexString("#990026")

  ###
  # Language
  ###
  if !$translateProvider.use()
    $translateProvider.use('de_DE')#Api.getExtendedLanguage())

  ###
  # Routing
  ###

  $stateProvider.state("tabs",
  {
    url: "/tabs"
    templateUrl: "templates/TabView.html"
    abstract: true
    controller: "TabCtrl"
  })
  .state("tabs.home",
  {
    # =~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
    # Home
    # =~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
    url: "/home"
    abstract: true
    views: "home":
      template: "<ion-nav-view></ion-nav-view>"
      # Home list view
    }).state("tabs.home.overview", {
      url: "/overview"
      templateUrl: "templates/HomeView.html"
      controller: "HomeCtrl"
      # Detail view of a question
    }).state("tabs.home.article", {
      url: "/article/?id/?page/?section/?create"
      templateUrl: "templates/ArticleView.html"
      controller: "ArticleCtrl"
      # Detail view of a question
    }).state("tabs.home.login", {
      url: "/login/?message"
      templateUrl: "templates/LoginView.html"
      controller: "LoginCtrl"
      # Detail view of a question
    }).state("tabs.home.disclaimer", {
      url: "/disclaimer"
      templateUrl: "templates/DisclaimerView.html"
      controller: "DisclaimerCtrl"
    }).state("tabs.home.promotion", {
      url: "/promotion"
      templateUrl: "templates/PromotionView.html"
      controller: "PromotionCtrl"
    }).state("tabs.home.help", {
      url: "/help"
      templateUrl: "templates/HelpView.html"
      controller: "HelpCtrl"
    }).state("tabs.home.important", {
      url: "/important"
      templateUrl: "templates/ImportantHintView.html"
      controller: "ImportantHintCtrl"
    })
  .state("tabs.favorites",
  {
    # =~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
    # Favorites
    # =~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
    url: "/favorites"
    abstract: true
    views: "favorites":
      template: "<ion-nav-view></ion-nav-view>"
      # favorites list view
    }).state("tabs.favorites.overview", {
      url: "/overview"
      templateUrl: "templates/FavoritesView.html"
      controller: "FavoritesCtrl"
      # Detail view of a question
    }).state("tabs.favorites.article", {
      url: "/article/?id/?page/?section"
      templateUrl: "templates/ArticleView.html"
      controller: "ArticleCtrl"
      # Detail view of a question
    }).state("tabs.favorites.login", {
      url: "/login/?message"
      templateUrl: "templates/LoginView.html"
      controller: "LoginCtrl"
      # Detail view of a question
    }).state("tabs.favorites.disclaimer", {
      url: "/disclaimer"
      templateUrl: "templates/DisclaimerView.html"
      controller: "DisclaimerCtrl"
    }).state("tabs.favorites.promotion", {
      url: "/promotion"
      templateUrl: "templates/PromotionView.html"
      controller: "PromotionCtrl"
    }).state("tabs.favorites.help", {
      url: "/help"
      templateUrl: "templates/HelpView.html"
      controller: "HelpCtrl"
    }).state("tabs.favorites.important", {
      url: "/important"
      templateUrl: "templates/ImportantHintView.html"
      controller: "ImportantHintCtrl"
    })
  .state("tabs.search",
  {
    # =~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
    # Search
    # =~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
    url: "/search"
    abstract: true
    views: "search":
      template: "<ion-nav-view></ion-nav-view>"
      # Search list view
    }).state("tabs.search.overview", {
      url: "/overview"
      templateUrl: "templates/SearchView.html"
      controller: "SearchCtrl"
      # Detail view of a question
    }).state("tabs.search.article", {
      url: "/article/?id/?page/?section"
      templateUrl: "templates/ArticleView.html"
      controller: "ArticleCtrl"
      # Detail view of a question
    }).state("tabs.search.login", {
      url: "/login/?message"
      templateUrl: "templates/LoginView.html"
      controller: "LoginCtrl"
      # Detail view of a question
    }).state("tabs.search.disclaimer", {
      url: "/disclaimer"
      templateUrl: "templates/DisclaimerView.html"
      controller: "DisclaimerCtrl"
    }).state("tabs.search.promotion", {
      url: "/promotion"
      templateUrl: "templates/PromotionView.html"
      controller: "PromotionCtrl"
    }).state("tabs.search.help", {
      url: "/help"
      templateUrl: "templates/HelpView.html"
      controller: "HelpCtrl"
    }).state("tabs.search.important", {
      url: "/important"
      templateUrl: "templates/ImportantHintView.html"
      controller: "ImportantHintCtrl"
    })
  .state("tabs.settings",
  {
    # =~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
    # Settings
    # =~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
    url: "/settings"
    abstract: true
    views: "settings":
      template: "<ion-nav-view></ion-nav-view>"
    }).state("tabs.settings.overview", {
      url: "/overview"
      templateUrl: "templates/SettingsView.html"
      controller: "SettingsCtrl"
    }).state("tabs.settings.privacy", {
      url: "/privacy"
      templateUrl: "templates/PrivacyView.html"
      controller: "PrivacyCtrl"
    }).state("tabs.settings.disclaimer", {
      url: "/disclaimer"
      templateUrl: "templates/DisclaimerView.html"
      controller: "DisclaimerCtrl"
    }).state("tabs.settings.promotion", {
      url: "/promotion"
      templateUrl: "templates/PromotionView.html"
      controller: "PromotionCtrl"
    }).state("tabs.settings.help", {
      url: "/help"
      templateUrl: "templates/HelpView.html"
      controller: "HelpCtrl"
    }).state("tabs.settings.important", {
      url: "/important"
      templateUrl: "templates/ImportantHintView.html"
      controller: "ImportantHintCtrl"
    })

  $urlRouterProvider.otherwise('/tabs/home/overview')

  return

###
# Ask module declartion
###

angular.module("Flexikon.services", [])
angular.module("Flexikon.controllers", [])
angular.module("Flexikon.directives", [])
angular.module("Flexikon.translations", ['pascalprecht.translate'])
angular.module("templates", [])

