angular.module("Flexikon.directives").directive('dcAd', ($timeout, $q, $rootScope, $sce, $http, $parse, $ionicGesture, Store, Api, GlobalSettings, InAppPurchase, UniqueID) ->
  restrict: 'E'
  scope:
    refreshEvent   : '@' # refresh ad on specified event
    noAdCallback   : '&' # callback in case no ad has been retrieved
    successCallback: '&' # callback in case ad has been loaded successfully
    devIdDe        : '@' # zone ID for development (DE)
    devIdEn        : '@' # zone ID for development (EN)
    prodIdDe       : '@' # zone ID for production (DE)
    prodIdEn       : '@' # zone ID for production (EN)
  template: ''
  link: (scope, element, attrs) ->

    ###*
     * [urlparams - parameters for retrieving ad from adserver]
    ###
    scope.urlparams =
      rand : parseInt(UniqueID.get())
      category : if ionic.Platform.isIOS() then 'ios' else 'android'

    ###*
     * [ad description]
    ###
    scope.ad =
      markup     : ''
      native     : false
      target     : ''
      error      : false
      lastRefresh: -1

    ###*
     * [id - zone IDs for retrieving ads from the ad server]
    ###
    scope.id =
      development :
        de : scope.devIdDe || '571'
        en : scope.devIdEn || '571'
      production  :
        de : scope.prodIdDe || '575'
        en : scope.prodIdEn || '577'

    ###*
     * [hide - hide completely (in case of premium or no ad retrieved)]
     * @return promise
    ###
    scope.hide = ->
      element.addClass 'hiding'

      $timeout ->
        element.removeClass 'hiding'
        element.addClass 'hidden'
      , 350
  
    ###*
     * [fade - fade out on hide]
     * @return promise
    ###
    scope.fade = ->
      return $q((resolve, reject) ->
        element.addClass 'hiding'
        $timeout ->
          resolve()
        , 350
      )

    ###*
     * [show - show ad (when loaded)]
     * @return
    ###
    scope.show = ->
      element.removeClass 'hidden'
      element.removeClass 'hiding'
      element.addClass 'showing'

      $timeout ->
        element.removeClass 'showing'
      , 350

    ###*
     * [loadAdvertisement - retrieve ad from adserver]
     * @return
    ###
    scope.loadAdvertisement = ->
      if scope.ad.lastRefresh != -1 && (Date.now() - scope.ad.lastRefresh) / 1000 / 60 / 60 < 1
        return

      scope.fade()
      scope.ad.error = false
      return $http({
        url: "http://adserver.doccheck.com/www/delivery/ajs.php?zoneid=" + scope.id[Api.getEnv()][GlobalSettings.Language()] + "&cb=" + scope.urlparams.rand + "&n=a906a3a4"
        method: "GET"
        cache: false
      }).success((result) ->

        if result.match(/href=\\'F\\'/g)? && result.match(/href=\\'F\\'/g).length > 0
          scope.hide()
          scope.noAdCallback?()
          return

        #s.innerHtml = result
        s = document.createElement('script')
        $(s).attr('ng-init', "log()")
        $(s).html(result.replace(/document\.write\(/gmi, 'var dcad = ('))
        element.append(s)

        dcad = window.dcad

        $timeout(->
          if dcad?.length == 0
            return

          if dcad.match(/a href=([^ ]*)/)?.length == 0
            return

          targetMatch = dcad.match(/a href=([^ ]*)/)
          if targetMatch?.length > 0
            scope.ad.target = dcad.match(/a href=([^ ]*)/)[1].substr(1).slice(0, -1)
            dcad = dcad.replace(/a href=([^ ]*)/, (match, s1) ->
              return match.replace(s1, '')
            )
          dcad = dcad.replace(/href="\/\//gmi, 'href="http://')

          ###
          iframe = document.createElement('iframe')
          element.append(iframe)
          frameDoc = iframe.document
          if iframe.contentWindow
            frameDoc = iframe.contentWindow.document
          frameDoc.open()
          frameDoc.writeln(dcad)
          frameDoc.close()
          ###

          $(element).html(dcad)
          scope.gestureHandler = $ionicGesture.on('tap', scope.openAd, element)
          scope.gestureHandler = $ionicGesture.on('click', scope.preventDefault, element)
          
          scope.successCallback?()
          scope.ad.lastRefresh = Date.now()

          #scope.fade().then(->
          document.removeEventListener("online", scope.loadAdvertisement, false)
        )
        
        ###
        try
          scope.ad.markup = result.replace(/"\+"/g,'')
            .match(/<a.*\/a>/)[0]
            .replace(/\\'/g, "'")
            .replace(/<img /, "<img ng-click='openAd($event)' ")
            .replace(/<a /, "<a ng-click='openAd($event)' ")
          scope.ad.target = scope.ad.markup.match(/href=([^ ]*)/)[1].substr(1).slice(0, -1)

          imgsrc = scope.ad.markup.match(/src=(?:'|")(.*?)(?:'|")/)[1]
          scope.ad.markup = scope.ad.markup.replace(/<a /, "<a style='background-image: url(" + imgsrc + ")'")

          scope.ad.lastRefresh = Date.now()
        catch
          scope.ad.error = true
          scope.noAdCallback?()
          scope.hide()
          return

        scope.successCallback?()
        scope.show()
        #)
        ###
      ).error((error) ->
        scope.ad.error = true
        scope.noAdCallback?()
        scope.hide()

        document.removeEventListener("online", scope.loadAdvertisement, false)
        document.addEventListener("online", scope.loadAdvertisement, false)
      )

    ###*
     * [openAd - follow ad link, open in system browser]
     * @param  {Event} $event [click event for stopping event propagation]
     * @return
    ###
    scope.openAd = ($event) ->
      $event.preventDefault()
      $event.stopPropagation()
      if scope.ad.target?.length > 0
        window.open(scope.ad.target, "_system")
      return

    ###*
     * [preventDefault]
     * @param  {[type]} $event []
     * @return {[type]}        []
    ###
    scope.preventDefault = ($event) ->
      $event.preventDefault()
      $event.stopPropagation()
      return

    ###*
     * [hide ads immediately when premium is purchased]
     * @return
    ###
    $rootScope.$on('InAppPurchase-succeeded', ->
      scope.hide()
    )

    ###*
     * [show if not owned]
     * @return
    ###
    $rootScope.$on('InAppPurchase-notowned', ->
      if !Store.isOwned()
        scope.show()
    )
    
    ###*
     * intial check for premium
    ###
    InAppPurchase.get('flexikon_offline')
    .then((doc) ->
      if doc.article.purchased
        scope.hide()
      else
        scope.loadAdvertisement()
        if scope.refreshEvent? && scope.refreshEvent.length > 0
          scope.$on(scope.refreshEvent, scope.loadAdvertisement)
    ).catch((doc) ->
      if !store?
        console.warn('store object is not available - currently testing in browser? Otherwise check the plugin!')
        return
      
      if !store.get('flexikon_offline').owned
        scope.loadAdvertisement()
    )

    scope.$on('$destroy', ->
      scope.gestureHandler?()
    )

)