angular.module("Flexikon.controllers").controller('DcFlexikonArticleCtrl', ($scope,
  $rootScope,
  $filter,
  $timeout,
  $controller,
  $ionicScrollDelegate,
  $ionicSideMenuDelegate,
  $ionicHistory,
  $state,
  $ionicPlatform,
  $cordovaVibration,
  $ionicPopover,
  $q,
  WikiHTML,
  Favorites,
  LastSeen,
  FlexikonApi,
  ToC,
  Theme,
  ArticleHistory,
  Load,
  Login,
  Store,
  StateChangeInterrupter,
  DcMediaWiki,
  DcDateTime,
  FontSize,
  DcHighlight,
  GuidedTour
) ->

  @state =
    isNew           : false
    isOffline       : false
    isFetching      : false
    isPurchased     : false
    showSearch      : false
    showHint        : false
    showOptions     : false
    showStyleOverlay: false
    summerbreak     :
      active          : undefined

  @articleState =
    edit           : false
    isUnsaved      : false
    isFavorite     : false
    hasSuccessor   : false

  @buttonState =
    showOptions   : false

  @find =
    model  : ''
    count  : 0
    current: 0

  @scrolling =
    last       : -1
    listen     : true
    inProgress : false
    onTop      : true
    interval   : 20
    currentStep: Date.now()

  @fontSize = FontSize.getCurrent()

  @listener = []

  $scope.vm = @

  ###*
   * [init ]
   * @return {[type]} []
  ###
  @init = =>
    #$scope.performSummerbreakCheck()

    $controller('DcFlexikonArticleModalCtrl',
      $scope: $scope
    )

    $ionicPopover.fromTemplateUrl('summerbreakPopover.html',
      scope: $scope
    ).then((popover) =>
      @state.summerbreak.popover = popover
    )

    @addFindInPageListener()

    @listener.push $scope.$on('article.enter', =>
      @fetchArticle().then(@isSummerbreak)
      ArticleHistory.push($state.current.name, @page || $state.params.page)

      @scrolling.listen = true
      @articleState.hasSuccessor = ArticleHistory.forwardAvailable($state.current.name)

      if @article?.toc?
        ToC.set(@article.toc)

      if @find?.model?.length > 0
        @find.count = DcHighlight.highlightWord($scope.getHTMLWrapper()[0], @find.model)
        if @find.current >= 0 && @find.current < @find.count
          DcHighlight.focus(@find.current)

      window.addEventListener('orientationchange', $scope.toggleBars)
      $scope.toggleBars()
    )

    @listener.push $scope.$on('article.leave', =>
      @scrolling.listen = false
      DcHighlight.removeHighlights($scope.getHTMLWrapper()[0])
      window.removeEventListener('orientationchange', $scope.toggleBars)
    )

    @listener.push $rootScope.$on("article:font-size-changed", (event, data) =>
      @setFontSize()
    )

    @listener.push $rootScope.$on("article:toggle-style-overlay", (event, data) =>
      @state.showStyleOverlay = !@state.showStyleOverlay
    )

    @listener.push $scope.$watch(=>
      return @fontSize
    , (n, o) ->
      FontSize.set(n)
    )

    @listener.push $scope.$watch(=>
      return @wikiModel
    , (n, o) =>
      if !n?
        return

      @adjustWikiContent()
    )

    $scope.$on('$destroy', =>
      for listener in @listener
        listener?()
    )
    return

  ###*
   *
   *
   * INITIAL LOADING & PROCESSING
   *
   *
  ###

  ###*
   * [fetchArticle]
   * @param  {[type]} page []
   * @return {[type]}      []
  ###
  @fetchArticle = (page) =>
    if @fetchPromise?
      return @fetchPromise

    if !page?
      page = @page || $state.params.page

    extracted = DcMediaWiki.getLanguageAndTitle(page)
    @language = extracted.language
    @page     = extracted.title

    @state.isFetching = true

    return @fetchPromise = FlexikonApi.getArticle(@page, @language)
    .then((response) =>
      @state.isOffline = false

      # empty article
      if Object.keys(response.data.query.pages)[0] == '-1'
        @prepareNewArticle(page)
        return

      # article found -> process it
      for k,v of response.data.query.pages
        article = v

      if article.revisions?[0]?['*']? && @article?.content?.wiki? && article.revisions[0]['*'] == @article.content.wiki
        return

      # catch redirect
      if (article.revisions[0]['*']).toUpperCase().match(/^(#REDIRECT)|(#WEITERLEITUNG)/gm)?.length > 0
        @fetchPromise = null;
        return @fetchArticle((article.revisions[0]['*']).match(/\[\[(.*)\]\]/)[1])

      @processArticle(article)
      @scrollToSection()
      @addToHistory()
      @fetchFavoriteState()
      @gaLogOpen()
      @setFontSize()

      if !@articleState.isNew
        @runOverlay()

      ###
      TODO:
        $scope.runOverlay()
      ###

    ).catch((error) =>
      @state.isOffline = true

      # if offline availability hasn't been purchased
      if !@isPremium()
        return

      # load from local DB if available
      @state.isPremium = true
      return @fetchOfflineArticle(page)

    ).finally(=>
      @state.isFetching = false
      @fetchPromise = undefined
    )

  ###*
   * [fetchOfflineArticle]
   * @param  {[type]} page []
   * @return {[type]}      []
  ###
  @fetchOfflineArticle = (page) =>
    if page.lastIndexOf(':') >= 0
      page = page.substr(page.lastIndexOf(':')+1)

    Favorites.get(page)
    .then((result) =>
      @state.isOffline = true

      @language             = result.article.language
      @setArticleMeta(result.article.title, result.article.article)
      @versionDate          = result.article.date

      @processArticle()

      ###
      $scope.timeouts.push $timeout(->
        $scope.runOverlay()
      , 1000)
      ###

    , (error) =>
      @state.noArticle = true
      @state.isFetching = false
    )

  ###*
   * [isPremium]
   * @return {Boolean} []
  ###
  @isPremium = ->
    return Store.isOwned('flexikon_offline')

  ###*
   * [setArticleMeta]
   * @param {[type]} title       []
   * @param {[type]} wikiContent []
  ###
  @setArticleMeta = (title, wikiContent) =>
    @article               =
      title                  : {}
      content                : {}
    @article.title.raw     = title
    @article.title.trimmed = title.replace(/\/|_| |'|"/g, '')
    @article.content.wiki  = wikiContent
    if !@article.content.wiki_save?
      @article.content.wiki_save = @article.content.wiki
    return

  ###*
   * [setFontSize]
  ###
  @setFontSize = ->
    angular.element('.html-content').css('font-size', FontSize.getCurrent() + 'px')
    return

  ###*
   * [addToHistory]
  ###
  @addToHistory = =>
    if @state.isNew
      return

    return LastSeen.get(@article.title.raw)
    .then((result) ->
      result.article.lastVisited = Date.now()
      return LastSeen.update(result)
    ).catch((result) =>
      return LastSeen.insert(@article.title.raw,
        title      : @article.title.raw
        language   : @language,
        lastVisited: Date.now()
      )
    )

  ###*
   * [prepareNewArticle]
   * @return {[type]} []
  ###
  @prepareNewArticle = (page) =>
    @state.showHint    = true
    @state.isNew       = true

    @processArticle({
      'title' : page
      'revisions'   : [
        '*' : '==Definition=='
      ]
    })

    promise = @state.summerbreak.promise || @isSummerbreak()
    promise.then(=>
      if @state.summerbreak.active
        @state.summerbreak.popover.show(
          target : $scope.getSelf().find('.article-title')[0]
        )
        return

      @toggleEdit()
    )
    return

  ###*
   * [isSummerbreak]
   * @return {Boolean} []
  ###
  @isSummerbreak = =>
    return $q((resolve, reject) =>
      if @state.summerbreak.active == true && $event?
        @state.summerbreak.popover?.show($event)
        return resolve()

      if @state.summerbreak.active == false
        return resolve()

      @state.summerbreak.promise = FlexikonApi.summerbreakCheck()
      .then((response) =>
        angular.extend(@state.summerbreak, response.data)
        # activity state
        @state.summerbreak.active = @state.summerbreak['summer-break'] || false

        if !@state.summerbreak.active
          return resolve()

        # start
        if @state.summerbreak['start-date']? && !isNaN(parseInt(@state.summerbreak['start-date']))
          @state.summerbreak.from = DcDateTime.getFormattedDate(new Date(@state.summerbreak['start-date'] * 1000))
        # end
        if @state.summerbreak['end-date']? && !isNaN(parseInt(@state.summerbreak['end-date']))
          @state.summerbreak.to = DcDateTime.getFormattedDate(new Date(@state.summerbreak['end-date'] * 1000))

        if @state.summerbreak.message?.length > 0
          console.debug(@state.summerbreak.message)
        else if @state.summerbreak.from? && @state.summerbreak.to?
          @state.summerbreak.message = $filter(
            'translate'
            )(
              'DC_ARTICLE.SUMMERBREAK.POPOVER_HINT_DATE',
                from: @state.summerbreak.from,
                to  : @state.summerbreak.to
            )
        else
          @state.summerbreak.message += $filter(
            'translate'
          )(
            'DC_ARTICLE.SUMMERBREAK.POPOVER_HINT_PLAIN'
          )

        if @articleState.edit
          @toggleEdit()

        if response.data == true
          $scope.ui.showWiki = !$scope.ui.showWiki
        else if $event?
          $scope.summerbreakPopover?.show($event)

        return resolve()

      )
      .catch((error) =>
        if @state.isOffline
          return reject()

        if !@state.summerbreak.active
          @state.summerbreak =
            message : $filter(
              'translate'
            )(
              'DC_ARTICLE.SUMMERBREAK.POPOVER_HINT_FAILURE'
            )
            error   : true

        reject()
      )
    )

  ###*
   * [discard]
   * @return {[type]} []
  ###
  @discard = =>
    @article.content.wiki = @wikiSave
    @articleState.isUnsaved = false
    if @articleState.edit
      @toggleEdit()
    return

  ###*
   * [save]
   * @param  {[type]} $event []
   * @return {[type]}        []
  ###
  @save = ($event) =>
    Load.isBlocked(true)
    @isSummerbreak($event)
    .then(=>
      if @state.isSummerbreak
        return

      q = $q.defer()
      if @articleState.isNew
        if analytics? && @article.title?.raw?
          analytics.trackEvent('Article', 'create', $scope.article.title.raw)
        @articleState.isNew = false
      else
        if analytics? && @article.title?.raw?
          analytics.trackEvent('Article', 'edit', @article.title.raw)

      @adjustWikiContent()

      FlexikonApi.post(@article.title.raw, @article.content.wiki, undefined, (result) =>
        $ionicScrollDelegate.scrollTop(false)

        if @articleState.edit
          @toggleEdit()

        if @articleState.isFavorite
          @toggleFavoriteState()
          @toggleFavoriteState()

        $timeout(=>
          Load.isBlocked(false)
          @wikiSave = angular.copy(@article.content.wiki)
          @articleState.isUnsaved = false
          StateChangeInterrupter.unregisterCallback()

          q.resolve()

          if ionic.Platform.isWebView()
            $cordovaToast.showLongCenter $filter('translate')('DC_ARTICLE_SAVED')
        )
      ,
      (error) ->
        $timeout(->
          Load.isBlocked(false)
          q.reject()

          if ionic.Platform.isWebView()
            $cordovaToast.showLongCenter $filter('translate')('DC_ARTICLE_SAVED_ERROR')
        )
      )

      return q.promise

    ).finally(->
      Load.isBlocked(false)
    )

  ###*
   * [fetchFavoriteState ]
   * @return {[type]} []
  ###
  @fetchFavoriteState = =>
    return Favorites.exists(@article.title.raw)
    .then((result) =>
      @articleState.isFavorite = result
    )

  ###*
   * [toggleFavoriteState]
   * @return {[type]} []
  ###
  @toggleFavoriteState = =>
    return Favorites.get(@article.title.raw).then((doc) =>
      return Favorites.delete(doc).then((result) =>
        @articleState.isFavorite = false
        $timeout.cancel(@articleState.favoritePulseTimeout)
        $scope.getSelf().find('.favorite-button').removeClass('pulse')
      )
    ).catch((error) =>
      return Favorites.insert(@article.title.raw,
        article : @article.content.wiki,
        title : @article.title.raw,
        language : @language,
        date: DcDateTime.getFormattedDate()
      ).then((result) =>
        @articleState.isFavorite = true
        $scope.getSelf().find('.favorite-button').addClass('pulse')
        @articleState.favoritePulseTimeout = $timeout(->
          $scope.getSelf().find('.favorite-button').removeClass('pulse')
        , 1000)
      )
    )


  ###*
   * [processArticle]
   * @return {[type]} []
  ###
  @processArticle = (article) =>
    if !(@article?.title?.raw?)
      @setArticleMeta(article.title, article.revisions[0]['*'])

    # parse to HTML
    parsedHTML = WikiHTML.toHTML(@article.content.wiki, $scope.language)
    # generate ToC
    # modifes HTML by adding section IDs
    tocResult = @generateToC(parsedHTML, @article.title.raw)

    @article.content.html     = tocResult.article
    @article.content.sections = DcMediaWiki.splitHtml(@article.content.html)
    @article.toc              = tocResult.toc
    return

  ###*
   * [generateToC]
   * @param  {[type]} html  []
   * @param  {[type]} title []
   * @return {[type]}       []
  ###
  @generateToC = (html, title) ->
    return ToC.generate(html, title)


  ###*
   *
   *
   * TRACKING
   *
   *
  ###

  ###*
   * [gaLogOpen]
   * @return {[type]} []
  ###
  @gaLogOpen = =>
    if !(analytics? && @article.title?.raw?)
      return

    analytics.trackEvent(
      'Article'
      'open'
      @article.title.raw
    )

  ###*
   *
   *
   * STATE CHANGES
   *
   *
  ###

  ###*
   * [back]
   * @return
  ###

  ###*
   * [openArticle]
   * @param  {String} page       []
   * @param  {String} section='' []
   * @return
  ###
  @openArticle = (page, section='') ->
    if @articleState.isUnsaved
      $scope.openSavePopup('article', page, section)
      return

    $scope.closeModal()
    page = page.replace(/&[^;]*;/g, '')

    console.log ( page )

    $state.go('^.article',
      page   : window.decodeURIComponent(page)
      section: section
    )
    return

  ###*
   * [back]
   * @return {[type]} []
  ###
  @back = =>
    if @articleState.edit
      @articleState.edit = false
      return

    if @articleState.isUnsaved
      $scope.openSavePopup('back')
      return

    ArticleHistory.back($state.current.name)
    $ionicHistory.backView().go()

  ###*
   * [forward]
   * @return {[type]} []
  ###
  @forward = =>
    return @openArticle(ArticleHistory.forward($state.current.name))

  ###*
   *
   *
   * SCROLLING, POSITIONING & SIDE MENU
   *
   *
  ###

  ###*
   * [scrollToSection]
   * @param  {[type]} section []
   * @return {[type]}         []
  ###
  @scrollToSection = (section, fails) =>
    if !section?
      section = $state.params.section

    if !section?
      return

    if !section.targetId?
      section = ToC.getSectionWithHeading(section)

    if !(section?.targetId?)
      return

    if fails > 10 then return

    if angular.element('#' + section.targetId).length == 0
      return $timeout(=>
        @scrollToSection(section, ++fails)
      , 500)

    return @anchorScroll(id)

  ###*
   * [expandAll]
   * @return {[type]} []
  ###
  @expandAll = =>
    expandableSections = document.querySelectorAll('#ArticleScroll-' + @article.title.trimmed + ' dc-flexikon-section dc-expandable')

    for section in expandableSections
      if section.id?
        $rootScope.$broadcast('dcexpandable' + section.id + '-open',
          preventDefault : true
        )

  ###*
   * [anchorScroll]
   * @param  {[type]} id []
   * @return {[type]}    []
  ###
  @anchorScroll = (id) =>
    expandId = angular.element('#' + id).closest('dc-expandable').attr('id')
    $rootScope.$broadcast('dcexpandable' + expandId + '-open')

    $timeout(=>
      targetElement = angular.element('#' + id)
      if !(targetElement?.length > 0)
        return
      top = targetElement.position().top - 12

      targetSection = targetElement.closest('dc-flexikon-section')
      if (targetSection?.length > 0)
        top += targetSection.position().top

      @getScroll().scrollTo(0, top, true)

      @closeSideMenu()

      $timeout(->
        ToC.setCurrentSection(expandId)
      , 500)
    )

  ###*
   * [getScroll]
   * @return {[type]} []
  ###
  @getScroll = ->
    return $ionicScrollDelegate.$getByHandle('ArticleScroll')

  ###*
   * [scrollTop]
   * @return {[type]} []
  ###
  @scrollTop = =>
    return @getScroll().scrollTop(false)

  ###*
   * [resize]
   * @return {[type]} []
  ###
  @resize = =>
    return @getScroll().resize()

  ###*
   * [closeSideMenu]
   * @return {[type]} []
  ###
  @closeSideMenu = ->
    if $ionicSideMenuDelegate.isOpenRight()
      $ionicSideMenuDelegate.toggleRight()
      return true
    return false

  ###*
   * [toggleSideMenu]
   * @return {[type]} []
  ###
  @toggleSideMenu = ->
    $ionicSideMenuDelegate.toggleRight()
    return

  ###*
   *
   *
   * OPTIONS
   *
   *
  ###

  ###*
   * [toggleOptions]
   * @return {[type]} []
  ###
  @toggleOptions = ($event) =>
    $event?.preventDefault()
    $event?.stopPropagation()

    @buttonState.showOptions = !@buttonState.showOptions
    return

  ###*
   * [hideOptions]
   * @return {[type]} []
  ###
  @hideOptions = =>
    @buttonState.showOptions = false
    return

  ###*
   * [onButtonHold]
   * @param  {[type]} $event []
   * @return {[type]}        []
  ###
  @onButtonHold = ($event) =>
    $event?.preventDefault()
    $event?.stopPropagation()

    $target = angular.element($event.currentTarget)
    $target.css('-webkit-transition-duration', '0ms')
    $target.css('transition-duration', '0ms')

    matrix = $target.css('transform')
    if !matrix.match('matrix')? || matrix.match('matrix').length == 0
      @buttonState.start =
        x : 0
        y : 0
    else
      transform = matrix.match(/\(.*?\)/gi)[0]
      transform = transform.substr(1, transform.length - 2)
      transform = transform.split(', ')
      @buttonState.start =
        x : parseFloat(transform[4])
        y : parseFloat(transform[5])

    @buttonState.movable = true

    $ionicSideMenuDelegate.canDragContent(false)
    return

  ###*
   * [onButtonRelease]
   * @param  {[type]} $event []
   * @return {[type]}        []
  ###
  @onButtonRelease = ($event) =>
    $event?.preventDefault()
    $event?.stopPropagation()

    @buttonState.movable = false

    $target = angular.element($event.currentTarget)

    matrix = $target.css('transform')

    if !matrix.match('matrix')? || matrix.match('matrix').length == 0
      return

    transform = matrix.match(/\(.*?\)/gi)[0]
    transform = transform.substr(1, transform.length - 2)
    transform = transform.split(', ')
    @buttonState.end =
      x : parseFloat(transform[4])
      y : parseFloat(transform[5])

    if @buttonState.end.x < (window.innerWidth / 2) * (-1)
      @buttonState.end.x = (window.innerWidth - 60) * (-1)
      $target.addClass('left')
    else
      @buttonState.end.x = 0
      $target.removeClass('left')

    bottomBorder = window.innerHeight - (tabs = 49) - (header = 44) - (button = 60)
    if @buttonState.end.y < 0
      @buttonState.end.y = 0
    else if @buttonState.end.y > bottomBorder
      @buttonState.end.y = bottomBorder

      if @state.showSearch
        @buttonState.end.y -= 48

    if @buttonState.end.y > bottomBorder - 120
      $target.addClass('bottom')
    else
      $target.removeClass('bottom')

    $target.css('-webkit-transition-duration', '300ms')
    $target.css('transition-duration', '300ms')

    $target.css(
      'transform', 'translate3D(' + @buttonState.end.x + 'px, ' + @buttonState.end.y + 'px, 100px)'
    )
    $target.css(
      '-webkit-transform', 'translate3D(' + @buttonState.end.x + 'px, ' + @buttonState.end.y + 'px, 100px)'
    )

    $ionicSideMenuDelegate.canDragContent(true)

    return

  ###*
   * [onScroll]
   * @return {[type]} []
  ###
  @onScroll = =>
    scroll = @getScroll()

    # return if scroll isn't ready
    if !scroll.getScrollPosition()?
      return

    # if change is not significant, return
    if Math.abs(@scrolling.last - scroll.getScrollPosition().top) < 100
      return

    # set new scroll position + determine if on top or not
    @scrolling.last = scroll.getScrollPosition().top
    if @scrolling.onTop != (@scrolling.last < 1000)
      $timeout =>
        @scrolling.onTop = (@scrolling.last < 1000)

    # prevent clicks while scrolling
    @preventClickOnScroll()
    # highlight the currently visible section in the ToC
    @findActiveSection()

    # if article isn't active anymore, stop the loop
    if !@scrolling.listen
      return

    # otherwise -> next turn!
    return ionic.DomUtil.requestAnimationFrame(@onScroll)

  ###*
   * [preventClickOnScroll]
   * @return {[type]} []
  ###
  @preventClickOnScroll = =>
    @scrolling.inProgress = true
    $timeout.cancel(@scrolling.preventClickTimeout)
    @scrolling.preventClickTimeout = $timeout(=>
      @scrolling.inProgress = false
    , 150, false)
    return

  ###*
   * [findActiveSection]
   * @return {[type]} []
  ###
  @findActiveSection = =>
    nearest =
      gap     : 100000
      section : undefined

    # get all the sections
    sections = $scope.getHTMLWrapper()[0].getElementsByTagName('dc-flexikon-section')

    # find the closest section
    for section in sections
      if Math.abs(section.offsetTop - @scrolling.last) < nearest.gap
        nearest.gap = Math.abs(section.offsetTop - @scrolling.last)
        nearest.section = angular.element(section)

    # if there is one, highlight it
    if nearest.section?
      expandable = nearest.section.find('dc-expandable')
      if expandable? && expandable.length > 0 && expandable.attr('id')?
        ToC.setCurrentSection(expandable.attr('id'))

    return

  ###*
   * [move]
   * @param  {[type]} $event []
   * @return {[type]}        []
  ###
  @onButtonDrag = ($event) =>
    if !@buttonState.movable
      return

    $ionicSideMenuDelegate.canDragContent(false)
    $event.preventDefault()
    $event.stopPropagation()

    x = @buttonState.start.x + $event.gesture.deltaX
    y = @buttonState.start.y + $event.gesture.deltaY

    angular.element($event.currentTarget).css(
      'transform', 'translate3D(' + x + 'px, ' + y + 'px, 100px)'
    )
    angular.element($event.currentTarget).css(
      '-webkit-transform', 'translate3D(' + x + 'px, ' + y + 'px, 100px)'
    )
    return

  ###*
   * [toggleFindInPage]
   * @return {[type]} []
  ###
  @toggleFindInPage = ($event, focus=true) =>
    $event?.preventDefault()
    $event?.stopPropagation()
    @hideOptions()
    DcHighlight.removeHighlights($scope.getHTMLWrapper()[0])
    @state.showSearch = !@state.showSearch
    window.getSelection().removeAllRanges()

    if !@state.showSearch
      $timeout(=>
        @find =
          model  : ''
          count  : 0
          current: 0
      , 150, false)
    else if focus
      $timeout(=>
        @focusFindInPage()
      , 300)
    return

  ###*
   * [focusFindInPage]
   * @return {[type]} []
  ###
  @focusFindInPage = ->
    input = $scope.getSelf().find('#findInPage')
    input.focus()
    input[0].focus()
    return

  ###*
   * [blurFindInPage]
   * @return {[type]} []
  ###
  @blurFindInPage = ->
    input = $scope.getSelf().find('#findInPage')
    input.blur()
    input[0].blur()
    return

  ###*
   * [toggleFindInPage]
   * @return {[type]} []
  ###
  @hideFindInPage = =>
    @state.showSearch = false
    return

  ###*
   * [addFindInPageListener]
  ###
  @addFindInPageListener = =>
    @removeFindInPageListener()
    @listener.push @findInPageListener = $scope.$watch(=>
      return @find.model
    , (n, o) =>
      if n == o || !o?
        return
      @find.count = 0
      @find.current = 0
      delete @find.searched

      $timeout.cancel(@find.debounce)
      @find.debounce = $timeout(=>
        if @find.model.length >= 3
          @findInPage(false, false, false)
      , 300)
    )
    return

  ###*
   * [removeFindInPageListener]
   * @return {[type]} []
  ###
  @removeFindInPageListener = =>
    @findInPageListener?()
    return

  ###*
   * [findInPage]
   * @return {[type]} []
  ###
  @findInPage = (inverse, focus=true, blur=true) =>
    $timeout.cancel(@find.debounce)

    if @find.model.length == 0
      @find.count = DcHighlight.removeHighlights($scope.getHTMLWrapper()[0])
      $timeout(@focusFindInPage)
      return

    @find.searched = true
    if @find.count == 0
      @find.count = DcHighlight.removeHighlights($scope.getHTMLWrapper()[0])
      @find.count = DcHighlight.highlightWord($scope.getHTMLWrapper()[0], @find.model)
      if @find.count > 0
        DcHighlight.focus(@find.current)
        @anchorScroll('dc-highlight-' + @find.current)
        if blur
          @blurFindInPage()
      else if focus
        $timeout(@focusFindInPage)
      return

    if !inverse
      @find.current = (@find.current + 1) % @find.count
    else if inverse
      @find.current = (@find.current + @find.count - 1) % @find.count

    DcHighlight.focus(@find.current)
    @blurFindInPage()

    @anchorScroll('dc-highlight-' + @find.current)

    return

  ###*
   * [scrollToSelection]
   * @return {[type]} []
  ###
  @scrollToSelection = =>
    if !(window.getSelection?()?)
      return

    selection = window.getSelection()
    if !(selection.baseNode?.parentNode?)
      return

    parentNode = angular.element(selection.baseNode.parentNode)
    top = parentNode.offset().top - 96

    @getScroll().scrollBy(0, top, true)

  ###*
   *
   *
   * EDIT
   *
   *
  ###

  ###*
   * [editSection]
   * @param  {[type]} $event        []
   * @param  {[type]} section       []
   * @param  {[type]} sectionNumber []
   * @return {[type]}               []
  ###
  @editSection = =>
    return ($event, section, sectionNumber) =>
      promise = @state.summerbreak.promise || @isSummerbreak()

      promise.then(=>
        if @state.summerbreak.active
          @state.summerbreak.popover.show($event)
          return

        preparedTitle = WikiHTML.toWiki("== *" + DcMediaWiki.escapeStringForRegex(section) + " *==")
        mReg = new RegExp(preparedTitle + '(?: |.)*((?!\\n==[^=])(.|\\n))*')
        match = @article.content.wiki.match(mReg)[0].replace(new RegExp('^' + section), '')

        @article.edit =
          section : match
          number  : sectionNumber

        return @toggleEdit($event)
      )

  ###*
   * [editGlobal]
   * @param  {[type]} $event []
   * @return {[type]}        []
  ###
  @editGlobal = ($event) =>
    promise = @state.summerbreak.promise || @isSummerbreak()

    promise.then(=>
      if @state.summerbreak.active
        @state.summerbreak.popover.show($event)
        return

      @toggleEdit($event)
    )
    return

  ###*
   * [adjustWikiContent]
   * @return {[type]} []
  ###
  @adjustWikiContent = =>
    if @article.edit?.section?
      @article.content.wiki = @article.content.wiki.replace(@article.edit.section, @wikiModel)
      @article.edit.section = @wikiModel
    else
      @article.content.wiki = angular.copy(@wikiModel)

    @articleState.isUnsaved = (@article.content.wiki != @article.content.wiki_save)

    if @articleState.isUnsaved
      StateChangeInterrupter.registerCallback($scope.openSavePopup)

  ###*
   * [toggleEdit]
   * @param  {Boolean} force=false []
   * @return
  ###
  @toggleEdit = ($event, force=false, section=undefined) =>
    if $event?
      $event.preventDefault()
      $event.stopPropagation()

    if !force && !Login.isAuthenticated()
      $state.go('^.login')
      return

    if @toggleDisabled
      return

    @toggleDisabled = true
    $timeout.cancel(@toggleTimeout)
    @toggleTimeout = $timeout(=>
      @toggleDisabled = false
    , 1000, false)

    @hideOptions()

    # toggling from edit to preview
    if @articleState.edit
      @adjustWikiContent()
      @articleState.edit = false
      @resize()
      @processArticle()
      return

    # toggling from article to edit
    if @article.edit?.section?
      @wikiModel = angular.copy(@article.edit.section)
    else
      @wikiModel = angular.copy(@article.content.wiki)
    @wikiSave          = angular.copy(@article.content.wiki)
    @articleState.edit = true
    @resize()
    return

  ###*
   * [handleClick]
   * @param  {[type]} $event []
   * @return {[type]}        []
  ###
  @handleClick = ($event) =>
    if @scrolling?.inProgress
      return

    element = $event.toElement

    if DcHighlight.isHighlightNode(element)
      element = element.parentNode

    if element.tagName.toUpperCase() == 'A' && element.href.indexOf(location.origin) != 0
      $event.preventDefault()
      $event.stopPropagation()

      window.open(element.href, '_system')
      return

    if element.tagName.toUpperCase() == 'A' && element.href.indexOf(location.origin) == 0
      $event.preventDefault()
      $event.stopPropagation()

      link = (element.href.substr(element.href.lastIndexOf('/') + 1))

      if element.href.indexOf('#') >= 0
        strippedLink = link.replace(/<.*?>/gmi, '')
        if link.indexOf('.html') > 0
          strippedLink = strippedLink.substr(strippedLink.indexOf('.html') + ('.html').length)

        if strippedLink.indexOf('#') == 0
          @anchorScroll(strippedLink.substr(1))
          return

      $scope.openModal($event, link, element.attributes.getNamedItem('section')?.value || '')

  ###*
   *
   * MODAL
   *
  ###

  ###*
   * [prepareModal]
   * @return {[type]} []
  ###
  @prepareModal = ->
    $ionicModal.fromTemplateUrl('articleLinkModal.html',
      scope: $scope
      animation: "slide-up-half fade"
    ).then((modal) ->
      @modal =
        handle  : modal
        title   : undefined
        section : undefined
        content : ''
        error   : false
    )

  ###*
   * [darkThemeSelected]
   * @return {Boolean} []
  ###
  @darkThemeSelected = ->
    return Theme.darkThemeSelected()

  @setTheme = (theme) ->
    return Theme.select(theme)

  ###*
   * GuidedTour
  ###

  ###*
   * [prepareOverlay]
   * @return {[type]} []
  ###
  @prepareOverlay = ->
    GuidedTour.setRootElement($scope.getSelf())
    return
    GuidedTour.start(
      'FlexikonArticleIntro',
      [
        selector : undefined
        hint : ''
      ]
    )
    return

  ###*
   * [removeOverlay]
   * @return {[type]} []
  ###
  @removeOverlay = ->
    GuidedTour.cancel()
    return

  ###*
   * [runOverlay]
   * @return {[type]} []
  ###
  @runOverlay = ->
    GuidedTour.setRootElement($scope.getSelf())
    if GuidedTour.isPassed('FlexikonArticleIntro')
      return

    GuidedTour.setRootElement($scope.getSelf())

    #scope.showBars()

    GuidedTour.start(
      'FlexikonArticleIntro',
      [
        selector : [ # Step 1) Add to favorites
          name  : '.favorite-button'
          shape : 'round'
        ,
          name  : 'a[icon="fa fa-heart"]'
          shape : 'rect'
          global: true
        ]
        hint : 'GUIDED_TOUR.ARTICLE_STEPS.NEW_BOOKMARK'
      ,
        selector : [ # Step 1) Add to favorites
          name  : '.dc-icon-button.options'
          shape : 'round'
        ]
        hint : 'GUIDED_TOUR.ARTICLE_STEPS.OPTIONS'
        callbacks :
          afterStep : =>
            @toggleOptions()
      ,
        selector : [ # Step 1) Add to favorites
          name  : '.dc-icon-button.search'
          shape : 'round'
        ]
        hint : 'GUIDED_TOUR.ARTICLE_STEPS.FIND_IN_PAGE_BUTTON'
        callbacks :
          afterStep : =>
            @toggleFindInPage(undefined, false)
      ,
        selector : [ # Step 1) Add to favorites
          name  : '.find-in-page .fa-close'
          shape : 'square'
        ,
          name   : '.find-in-page input'
          shape  : 'square'
          noArrow: true
        ,
          name  : '.article-title'
          shape : 'square'
        ]
        hint : 'GUIDED_TOUR.ARTICLE_STEPS.FIND_IN_PAGE_INPUT'
        callbacks :
          onHighlight : =>
            typeTitleInSearch = =>
              if @find.model.length == @article.title.raw.length
                return $q.resolve()

              return $timeout(=>
                @find.model += @article.title.raw[@find.model.length]
                typeTitleInSearch()
              , 100)

            return typeTitleInSearch()
            .then(=>
              return $timeout(=>
                @findInPage()
              , 300)
            )
          afterStep : =>
            @toggleFindInPage()
            @toggleOptions()
      ,
        selector: [
          name  : '.dc-icon-button.edit'
          shape : 'round'
        ]
        hint : 'GUIDED_TOUR.ARTICLE_STEPS.EDIT'
        callbacks :
          afterStep : =>
            @toggleEdit(undefined, true)
            @toggleOptions()
      ,
        selector : [
          name  : '.dc-icon-button.preview'
          shape : 'round'
        ]
        hint : 'GUIDED_TOUR.ARTICLE_STEPS.PREVIEW'
        callbacks :
          onFadeOut : =>
            $timeout(=>
              @toggleEdit(undefined, true)
            , 150)
            return
        buttonText: 'GUIDED_TOUR.GO_FOR_IT'
        disableCancel: true
      ],
      false
    )

  @init()

  return



)