angular.module("Flexikon.controllers").controller('DcFlexikonArticleModalCtrl', ($scope, $state, $filter, $ionicSideMenuDelegate, $ionicModal, $ionicHistory, $ionicTabsDelegate, $ionicPopup, WikiHTML, FlexikonApi, StateChangeInterrupter, ArticleHistory) ->

  ###*
   * [Logic for the link definition modal - in case an internal link is triggered, the definition will show up in the modal offering to move on to the whole article]
   * @return {[type]} [description]
  ###
  ionic.Platform.ready(->
    cordova?.plugins?.Keyboard?.disableScroll(true)

    $ionicModal.fromTemplateUrl('articleLinkModal.html',
      scope: $scope
      animation: "slide-up-half fade"
    ).then((modal) ->
      $scope.modal =
        handle  : modal
        title   : undefined
        section : undefined
        content : ''
        error   : false
    )

    ###*
     * [openModal - show the definition modal]
     * @param  {[Event]}  $event  [trigger click event]
     * @param  {[String]} page    [page string]
     * @param  {[String]}    section [section to open]
     * @return
    ###
    $scope.openModal = ($event, page, section) ->
      ###*
       * if this defintion has been opened before, don't fetch once again, just reopen it and return
      ###
      if $scope.modal.target == page
        $scope.modal.section = section
        $scope.modal.handle.show $event
        return

      ###*
       * reset all ui values to clean the modal before fetching a new definition
      ###
      $scope.modal.page = undefined
      $scope.modal.target = undefined
      $scope.modal.section = undefined
      $scope.modal.content = ''

      ###*
       * retrieve the page and the target
       * [target - string to identify the page]
       * [page   - formatted string to show as modal title]
      ###
      $scope.modal.page = $scope.modal.target = decodeURIComponent(page)
      $scope.modal.page = $scope.modal.page.replace('_', ' ')
      $scope.modal.section = section

      ###*
       * target might contain a language prefix, so fetch it and proceed
      ###
      page = $scope.modal.target
      if page.match(/^(:|\/)/)?.length > 0
        match = page.match(/(?::|\/)(.*)(?::|\/)/)
        page = page.replace(match[0], '').replace('_', ' ')
        language = match[1]
        $scope.modal.page = $scope.modal.target = decodeURIComponent(page)
        if $scope.modal.target.indexOf(':') != 0
          $scope.modal.target = ':' + language + ':' + $scope.modal.target
      ###*
       * no language given? So assume we keep the language of the currently shown article
      ###
      if !language then language = $scope.vm.language

      ###*
       * fetch the definition from the API
      ###
      FlexikonApi.getDefinition(page, language)
      .success((result) ->
        $scope.modal.content = WikiHTML.toHTML(result.text).replace(/<br *\/?>/, ' ').replace(/<(.*?)>/gmi, '')
        ###*
         * redirects might have occured - in case, check for the correct title to show as modal title
        ###
        if result.redirect_title? then $scope.modal.page = $scope.modal.target = decodeURIComponent(result.redirect_title)
        if $scope.modal.target.indexOf(':') != 0
          $scope.modal.target = ':' + language + ':' + $scope.modal.target
      ).error((data) ->
        ###*
         * error handling (404 -> not found; else -> unknown)
        ###
        $scope.modal.error = true
        if data.error.code == 404
          $scope.modal.new = true
          $scope.modal.content = $filter('translate')('DC_ARTICLE_NOT_FOUND')
        else
          $scope.modal.content = $filter('translate')('DC_ARTICLE_UNKNOWN_ERROR')
      )

      ###*
       * just make sure the side menu is closed
      ###
      if $ionicSideMenuDelegate.isOpenRight() then $ionicSideMenuDelegate.toggleRight()

      ###*
       * trigger the show function of the actual modal handle
      ###
      $scope.modal.handle.show $event
      return

    ###*
     * [openModalArticle - move on to the article the definition is shown for (if fetched successfully)]
     * @param  {[String]} page    [title of the article to open]
     * @param  {[String]} section [section to open (optional)]
     * @return
    ###
    $scope.openModalArticle = (page, section) ->
      if $scope.modal.error && !$scope.modal.new
        return
      $scope.vm.openArticle(page, section)

    ###*
     * [closeModal]
     * @return
    ###
    $scope.closeModal = ->
      $scope.modal.handle.hide()

      if $ionicSideMenuDelegate.isOpenRight() then $ionicSideMenuDelegate.toggleRight()
      return

    ###*
     * [remove the modal completely when the article is destroyed]
     * @return
    ###
    $scope.$on '$destroy', ->
      $scope.modal.handle.remove()
      return

    ###*
     * [reset the modal when hidden and fetching wasn't successful; otherwise keep the information in case the user opens the same definition again]
     * @return
    ###
    $scope.$on 'modal.hidden', ->
      if $scope.modal.error
        $scope.modal.error = false
        $scope.modal.page = undefined
        $scope.modal.target = undefined
        $scope.modal.section = undefined
        $scope.modal.content = ''
      return

    ###*
     * [modal removed event]
     * @return
    ###
    $scope.$on 'modal.removed', ->
      return
  )

  ###*
   * [openSavePopup]
   * @param  {[type]} type [description]
   * @param  {[type]} a    [description]
   * @param  {[type]} b    [description]
   * @return {[type]}      [description]
  ###
  $scope.openSavePopup = (type, a, b) ->
    $ionicPopup.confirm(
      title: $filter('translate')('DC_ARTICLE_LEAVE_HINT')
      cssClass:'dc-popup save-popup'
      buttons: [
        text: $filter('translate')('DC_ARTICLE_LEAVE_WITH_SAVE'),
        type: 'dc-button-block dc-button-red',
        onTap: (e) ->
          return 'save'
      ,
        text: $filter('translate')('DC_ARTICLE_LEAVE_WITHOUT_SAVE'),
        type: 'dc-button-block dc-button-gray',
        onTap: (e) ->
          return 'leave'
      ,
        text: $filter('translate')('DC_ARTICLE_CANCEL_LEAVE'),
        type: 'dc-button-block dc-button-gray',
        onTap: (e) ->
          return 'cancel'
      ]
    ).then((res) ->
      leave = ->
        StateChangeInterrupter.unregisterCallback()
        if type == 'back'
          ArticleHistory.back($state.current.name)
          $ionicHistory.goBack()
        else if type == 'article'
          $scope.vm.openArticle(a, b)
        else if type == 'state'
          $state.go(a)
          $ionicTabsDelegate.select(b)

      if res == 'leave'
        $scope.vm.discard()
        leave()
      else if res == 'save'
        $scope.vm.save().then(->
          leave()
        )
      else if res == 'cancel'
        x = x
      return
    )

)