angular.module("Flexikon.controllers").controller('DcFlexikonFavListingCtrl', ($scope, $rootScope, $controller, $q, $timeout, $location, $ionicScrollDelegate, Favorites, ExtendedState, LastSeen, FlexikonApi, Login, GlobalSettings) ->

	$controller('BaseCtrl', {$scope: $scope});

	###*
	 * [ui - ui state variables]
	###
	$scope.ui =
		loading : false
		contribsLoading : false

	###*
	 * [render - render state variables depending on loading status of sections]
	###
	$scope.render =
		lastseen 			: true
		favorites 		: true
		contributions : true

	###*
	 * [init]
	###
	$scope.$on('favs.beforeEnter', ->
		$scope.fetch()
		$scope.getChannelListing()
	)

	###*
	 * [fetch - fetch all resources from local dbs]
	 * @return 
	###
	$scope.fetch = ->		
		$scope.ui.loading = true
		$q.all([
			$scope.fetchFavorites(),
			$scope.fetchLastSeen()
		]).then(->
			$timeout(->
				$scope.ui.loading = false
			)
		)

	###*
	 * [fetchFavorites - fetch favorites from local db]
	 * @return
	###
	$scope.fetchFavorites = ->
		return $q((resolve, reject) ->
			Favorites.getAll().then((result) ->
				$scope.favorites = result
				resolve()
				$timeout(->
					$scope.render.favorites = true
				, 100)
			).catch((error) ->
				$scope.favorites = []
				reject()
			)
		)

	###*
	 * [fetchLastSeen - fetch last seen articles from local db]
	 * @return
	###
	$scope.fetchLastSeen = ->
		return $q((resolve, reject) ->
			LastSeen.getAll().then((result) ->
				$scope.lastSeen = result
				resolve()
				$timeout(->
					$scope.render.lastseen = true
				, 100)
			).catch((error) ->
				$scope.lastSeen = []
				reject()
			)
		)

	###*
	 * [getChannelListing - fetch articles of current user]
	 * @return 
	###
	$scope.getChannelListing = ->
		if !$scope.isAuthenticated()
			return

		$scope.ui.contribsLoading = true
		FlexikonApi.getChannelListing()
		.success((result) ->
			document.removeEventListener("online", $scope.getChannelListing, false)

			$scope.contribs = []
			contribsTitle = []
			for c in result.query.usercontribs
				if c.title not in contribsTitle
					contribsTitle.push c.title
					$scope.contribs.push c
			$scope.ui.contribsLoading = false
			$timeout(->
				$scope.render.contributions = true
			, 100)
		).error((error) ->
			console.warn error

			document.removeEventListener("online", $scope.getChannelListing, false)
			document.addEventListener("online", $scope.getChannelListing, false)

			$scope.ui.contribsLoading = false
			$scope.render.contributions = true

			if $scope.contribs?.length > 0
				return 

			$scope.contribs = []			
			$scope.ui.contribsError = true
		)

	###*
	 * [closeAllBut - close the other expandables when one is opened]
	 * @param  {String} id [clicked expandable id]
	 * @return {[type]}
	###
	$scope.closeAllBut = (id) ->
		$rootScope.$broadcast('dcexpandable-close-all-but', {id : id})

	###*
	 * [openArticle - open article]
	 * @param  {[type]} page     [page title]
	 * @param  {[type]} language [article language]
	 * @return {[type]}          
	###
	$scope.openArticle = ($event, page, language) ->
		$event.preventDefault()
		$event.stopPropagation()
		if !language? then language = GlobalSettings.Language()
		page = page.replace(/&[^;]*;/g, '')
		ExtendedState.openInSameBranch('article', {page : ':' + language + ':' + window.decodeURIComponent(page)})	

	$scope.fetch()	
	$scope.getChannelListing()	

)