angular.module("Flexikon.controllers").controller('DcFlexikonHomeListingCtrl', ($scope, $state, $ionicLoading, $cordovaToast, $ionicScrollDelegate, $timeout,  $filter, $rootScope, FlexikonApi, Api, Store) ->

  ###*
   * [ui - state variables]
  ###
  $scope.ui =
    loading    : false
    lastRefresh: -1

  ###*
   * [home - state variables for listing]
  ###
  $scope.home =
    items    : []
    show     : 6
    nomore   : false
    selection: []
    offset   : 0

  ###*
   * [ad - include ads regularly]
   * removed
  $scope.ad =
    offset   : 1
    interval : 7
  ###
  
  ###*
   * [scrollOnOrientationChange snapScroll after orientation change]
   * @return {[type]} [description]
  ###
  $scope.scrollOnOrientationChange = ->
    $scope.ui.automaticScroll = true
    $scope.timeouts.push $timeout(->
      if !$ionicScrollDelegate.$getByHandle('HomeScroll')?.getScrollPosition()?
        return
      $scope.snapScroll()
    , 500)
  window.addEventListener('orientationchange', $scope.scrollOnOrientationChange)
  window.addEventListener('resize', $scope.scrollOnOrientationChange)


  ###*
   * [watchers]
  ###
  $scope.$on('home.leave', ->
    window.removeEventListener('orientationchange', $scope.scrollOnOrientationChange)
    window.removeEventListener('resize', $scope.scrollOnOrientationChange)
  )

  $scope.$on('home.enter', ->
    window.addEventListener('orientationchange', $scope.scrollOnOrientationChange)
    window.addEventListener('resize', $scope.scrollOnOrientationChange)
  )

  $scope.$on('$destroy', ->
    window.removeEventListener('orientationchange', $scope.scrollOnOrientationChange)
    window.removeEventListener('resize', $scope.scrollOnOrientationChange)
  )

  $scope.$on("home.beforeEnter", ->
    $scope.home.offset = ($scope.home.offset + 3) % $scope.home.items.length

    for i in [$scope.home.offset..$scope.home.offset+2]
      $scope.home.selection.unshift($scope.home.selection.pop())
      #$scope.home.selection.push($scope.home.items[i % $scope.home.items.length])

    #$scope.home.selection = $scope.home.items.slice($scope.home.offset, $scope.home.offset + 4)

    refresh =  $scope.ui.lastRefresh < 0 || (Date.now() - $scope.ui.lastRefresh) / 1000 / 60 / 60 > 24
    
    # DEBUG: In Dev mode refresh each minute
    if Api.getEnv().indexOf('development') >= 0
      if ionic.Platform.isWebView() then $cordovaToast.showShortBottom 'DEBUGGING - In production refresh interval is set to ' + 300 + ' second(s).'
      refresh =  $scope.ui.lastRefresh < 0 || (Date.now() - $scope.ui.lastRefresh) / 1000 > 300

    if refresh then $scope.getArticles()
  )

  $rootScope.$on('InAppPurchase-succeeded', ->
    $scope.getArticles()
  )

  $scope.$on("$stateChangeSuccess", (event, toState, toParams, fromState, fromParams) ->
    if fromState.name.indexOf('home') < 0 && toState.name.indexOf('home') >= 0
      $ionicScrollDelegate.scrollTop(true)
  )

  ###*
   * [getArticles - get articles for listing]
   * @return
  ###
  $scope.getArticles = ->
    if $scope.home.items? && $scope.home.items.length > 0
      itemsSave = $scope.home.items.splice()
    $scope.ui.loading = true
    
    $scope.home.items = []

    FlexikonApi.getTopListing(200)
    .success((result) ->
      
      $scope.ui.lastRefresh = Date.now()
      
      if $scope.home?.items?
        $scope.home.items = []
        $timeout(->
          
          $scope.home.items = result._items.flexikon_top_articles
          $scope.home.offset = 0
          #$scope.home.selection = $scope.home.items.slice($scope.home.offset, 4)
          $scope.home.selection = $scope.home.items
          ###
          if !$scope.isOfflineVersion()
            for i in [0..$scope.home.items.length]
              if (i + $scope.ad.offset) % $scope.ad.interval == 0
                $scope.home.items.splice(i, 0, {
                  ad: 'ad'
                  index: i
                  title: 'ad' + i
                })
          ###

          $timeout ->
            $ionicScrollDelegate.$getByHandle('HomeScroll').resize()
                
          $scope.ui.loading = false
        )
      else
        $scope.home.items = result._items.flexikon_top_articles
        $timeout ->
          $ionicScrollDelegate.$getByHandle('HomeScroll').resize()
        $scope.ui.loading = false
      document.removeEventListener("online", $scope.getArticles, false)
    )
    .error((error) ->
      if itemsSave?
        $scope.home.items = itemsSave
        $timeout ->
          $ionicScrollDelegate.$getByHandle('HomeScroll').resize()
      $scope.ui.loading = false
            
      document.removeEventListener("online", $scope.getArticles, false)
      document.addEventListener("online", $scope.getArticles, false)
    )
  $scope.getArticles()

  ###*
   * [loadMore - infinite load]
   * @return
  ###
  $scope.loadMore = ->
    if $scope.home.nomore then return

    if $scope.home.show + 6 >= $scope.home.items.length
      $scope.home.show = $scope.home.items.length
      $scope.home.nomore = true
    else
      $scope.home.show += 6

    $scope.$broadcast('scroll.infiniteScrollComplete')

  ###*
   * [isOfflineVersion - check if is premium]
   * @return {Boolean} [is premium version]
  ###
  $scope.isOfflineVersion = ->
    return Store.isOwned('flexikon_offline')

  $scope.screenshotFilter = (article) ->
    return article.title == 'Influenza' || article.title == 'Leber' || article.title == 'Pollen'

  ###*
   * [openArticle - open article from listing]
   * @param  {[String]} page [title of page to open]
   * @return
  ###
  $scope.openArticle = (page) ->
    if !page?
      return

    page = page.replace(/&[^;]*;/g, '').replace(/<(.*?)>/g, '')
    $state.go('^.article', page : window.decodeURIComponent(page))

    #ExtendedState.openInSameBranch('article', {page : window.decodeURIComponent(page)})
  
)