angular.module("Flexikon.controllers").controller('DcGuidedTourCtrl', ($scope, $timeout, $interval, $window, $filter, $ionicPlatform, GuidedTour, $rootElement, $q) ->

  @KAPPA = .5522848

  @STATES =
    FADE_IN : 0
    FADE_OUT: 1
    SHOW    : 2
    HIDE    : 3
    PAUSE   : 4
    WAITING : 5

  @debug = false

  @config =
    # line style
    line      :
      color     : 'rgba(255,255,255,0.7)'
      width     : 3
    # arrow head style
    arrowHead :
      color     : '255,255,255'
      opacity   : '0.7'
      width     : 12
      height    : 12
    # backdrop
    backdrop  :
      enabled   : true
      color     : '0,0,0'
      opacity   : '0.7'
    # animation parameters
    animation :
      fps       : 30
      shrink    : true
      length    : 500

  @shown   = false
  @running = false

  @state        = @STATES.PAUSE
  @frame        = 0
  @framesPassed = 0

  @step = undefined


  ###*
   * [init]
   * @return {[type]} []
  ###
  @init = =>

    @timeouts = []

    GuidedTour.registerShowCallback(@show)
    GuidedTour.registerHideCallback(@hide)
    GuidedTour.registerCancelCallback(@cancel)

  ###*
   *    ______      __             _       __   ______                 __  _
   *  /_  __/_  __/ /_____  _____(_)___ _/ /  / ____/_  ______  _____/ /_(_)___  ____  _____
   *   / / / / / / __/ __ \/ ___/ / __ `/ /  / /_  / / / / __ \/ ___/ __/ / __ \/ __ \/ ___/
   *  / / / /_/ / /_/ /_/ / /  / / /_/ / /  / __/ / /_/ / / / / /__/ /_/ / /_/ / / / (__  )
   * /_/  \__,_/\__/\____/_/  /_/\__,_/_/  /_/    \__,_/_/ /_/\___/\__/_/\____/_/ /_/____/
  ###

  ###*
   * [show]
   * @return {[type]} []
  ###
  @show = (config={}) =>

    @setDimensions()

    window.addEventListener('orientationchange', @setDimensions)

    @canvas = document.createElement('canvas')

    @canvas.width = @width
    @canvas.height = @height

    scanvas = document.querySelector('canvas')
    if canvas?
      @sctx   = scanvas.getContext("2d")
      @ctx    = canvas.getContext("2d")

    @state = @STATES.SHOW
    @resetState()
    @shown = true

    angular.extend(@config, config)
    @deregisterBackButtonAction = $ionicPlatform.registerBackButtonAction(->
      console.log 'disabled'
    , 600)

    #@showText(0)
    @scaleTextOut()
    @step = undefined
    @steps = GuidedTour.getSteps()
    @root = GuidedTour.getRootElement() || @root
    @root?.addClass?('guided-tour')

    @startDrawLoop()

    return

  ###*
   * [setDimensions]
  ###
  @setDimensions = =>
    @height = $window.innerHeight
    @width = $window.innerWidth
    return

  ###*
   * [startFadeIn]
   * @return {[type]} []
  ###
  @startFadeIn = =>
    @state = @STATES.FADE_IN
    return

  ###*
   * [startDrawLoop]
   * @return {[type]} []
  ###
  @startDrawLoop = =>
    if !@lastFrame?
      ionic.DomUtil.requestAnimationFrame(@draw)
    return

  ###*
   * [stopDrawLoop]
   * @return {[type]} []
  ###
  @stopDrawLoop = =>
    @state = @STATES.PAUSE
    return

  ###*
   * [getNextStep]
   * @return {[type]} []
  ###
  @getNextStep = =>
    afterStepPromise = @step?.callbacks?.afterStep?() || $q.resolve()
    afterStepPromise.then(=>
      if @steps.length == 0
        return undefined

      @step = @steps.splice(0, 1)[0]
    ).then(=>
      return @step?.callbacks?.beforeStep?() || $q.resolve()
    ).then(=>
      return $q((resolve) =>
        ionic.DomUtil.requestAnimationFrame(=>
          resolve(@step)
        )
      )
    )

  ###*
   * [draw]
   * @return {[type]}      []
  ###
  @draw = =>
    if @lastFrame? && Date.now() - @lastFrame < @getIntervalDuration()
      return ionic.DomUtil.requestAnimationFrame(@draw)

    # STATE PAUSED
    if (@state == @STATES.PAUSE || @state == @STATES.WAITING) && @frame % @config.animation.fps <= 1
      @frame = 0
      return undefined

    if !@lastFrame?
      @frame = (@frame + 1) % @config.animation.fps
    else
      steps = parseInt((Date.now() - @lastFrame) / @getIntervalDuration())
      if @config.animation.fps - steps < steps
        @frame = (@frame + steps) % @config.animation.fps
      else
        @frame = (@frame + 1) % @config.animation.fps

    @lastFrame = Date.now()

    # STATE SHOW
    if @state == @STATES.SHOW

      if @frame == 0
        @state = @STATES.PAUSE
        return ionic.DomUtil.requestAnimationFrame(=>
          return @getNextStep().then(=>
            @state = @STATES.FADE_IN
            @setText(@step.hint)
            @resetState()
            @startDrawLoop()
          )
        )

      currentOpacity = parseFloat(@config.backdrop.opacity) * @getIntervalProgress()
      @clearCanvas(@config.backdrop.color, currentOpacity)

    # STATE HIDE
    else if @state == @STATES.HIDE

      if @frame <= 1
        @hide()
        return ionic.DomUtil.requestAnimationFrame(@draw)

      currentOpacity = parseFloat(@config.backdrop.opacity) * (1 - @getIntervalProgress())
      @clearCanvas(@config.backdrop.color, currentOpacity)

    # STATE FADE_IN
    else if @state == @STATES.FADE_IN
      @fadeStepIn()

    # STATE FADE_OUT
    else if @state == @STATES.FADE_OUT
      @fadeStepOut()

    @framesPassed++

    @sctx.drawImage(@canvas, 0, 0)
    return ionic.DomUtil.requestAnimationFrame(@draw)

  ###*
   * [resetState]
   * @return {[type]} []
  ###
  @resetState = =>
    @framesPassed = 0
    @frame = 0
    @lastFrame = undefined
    return

  ###*
   * [proceedToNextStep]
   * @return {[type]} []
  ###
  @proceedToNextStep = ($event) =>
    $event?.preventDefault()
    $event?.stopPropagation()

    @lastFrame = undefined
    if @state != @STATES.WAITING
      return

    promise = @step?.callbacks?.onFadeOut?() || $q.resolve()

    promise.then(=>
      @state = @STATES.FADE_OUT
      @startDrawLoop()
    )
    return

  ###*
   * [fadeStepIn]
   * @return {[type]} []
  ###
  @fadeStepIn = =>
    if @frame == 0
      @state  = @STATES.PAUSE
      promise = @step?.callbacks?.onHighlight?() || $q.resolve()

      promise.then(=>
        @frame = @config.animation.fps - 1
        if @step?
          @highlightElements(@step.selector, @config.animation.fps, false)
        @resetState()
        @state = @STATES.WAITING
      )
      return

    @clearCanvas()
    if @step?
      @highlightElements(@step.selector, @config.animation.fps, false)
      if @step.hint?
        @setText(@step.hint)

    #@showText(@getIntervalProgress())
    @scaleTextIn()

  ###*
   * [fadeStepOut]
   * @return {[type]} []
  ###
  @fadeStepOut = =>
    if @frame == 0
      if @steps.length == 0
        @state = @STATES.HIDE
        @step?.callbacks?.afterStep?()
        GuidedTour.setPassed()
      else
        @state = @STATES.PAUSE
        ionic.DomUtil.requestAnimationFrame(=>
          @getNextStep().then(=>
            @state = @STATES.FADE_IN
            @resetState()
            @startDrawLoop()
          )
        )
      @resetState()
      return

    @scaleTextOut()
    #@showText(1 - @getIntervalProgress())
    @clearCanvas()
    @highlightElements(@step.selector, @config.animation.fps, true)

  ###*
   * [highlightElements ]
   * @param  {[type]} elements=[] []
   * @param  {[type]} duration    []
   * @param  {[type]} fade=false  []
   * @return {[type]}             []
  ###
  @highlightElements = (elements = [], duration, fade=false) =>
    progress = @getIntervalProgress()
    if fade
      progress = 1 - progress
      if @steps.length == 0
        @clearCanvas('0,0,0', progress * parseFloat(@config.backdrop.opacity))

    for element in elements
      @highlightElement(element, duration, fade)

    if @framesPassed >= duration
      @state = @STATES.WAITING
      @resetState()
    return

  @showText = (progress) ->
    $('#guided-tour #hint').css('opacity', progress)


  @scaleTextIn = ->
    $('#guided-tour #hint').css('-webkit-transform', 'scale3D(1, 1, 1)')
    $('#guided-tour #hint').css('transform', 'scale3D(1, 1, 1)')

  @scaleTextOut = ->
    $('#guided-tour #hint').css('-webkit-transform', 'scale3D(0, 1, 1)')
    $('#guided-tour #hint').css('transform', 'scale3D(0, 1, 1)')

  ###*
   * [highlightElement]
   * @param  {[type]} element  []
   * @param  {[type]} duration []
   * @param  {[type]} fade     []
   * @return {[type]}          []
  ###
  @highlightElement = (element, duration, fade) =>
    if element.global
      domElement = angular.element(element.name)
    else
      domElement = @root.find(element.name)
    if domElement.length == 0
      return

    progress = @getIntervalProgress()
    if fade
      progress = 1 - progress

    @prepareBrush('rgba(255,255,255,' + progress + ')', 'destination-out')

    if element.shape == 'round'
      @drawEllipse(
        domElement[0].getBoundingClientRect().left-10,
        domElement[0].getBoundingClientRect().top-10,
        domElement[0].offsetWidth+20,
        domElement[0].offsetHeight+20,
        10
      )
    else
      @drawRoundedRectangle(
        domElement[0].getBoundingClientRect().left-10,
        domElement[0].getBoundingClientRect().top-10,
        domElement[0].offsetWidth+20,
        domElement[0].offsetHeight+20,
        10
        1
      )

    if !fade && !@hideArrows() && !element.noArrow
      @drawArrowToElement(domElement, progress, fade && !@config.animation.shrink)

  ###*
   * [hideArrows]
   * @return {Boolean} []
  ###
  @hideArrows = ->
    return ((
      @width > @height && @height < 768
    ) || (
      @height > @width && @height <= 480
    ))

  ###*
   * [getIntervalDuration]
   * @return {[type]} []
  ###
  @getIntervalDuration = =>
    return 1000 / @config.animation.fps * @config.animation.length / 1000

  ###*
   * [getIntervalProgress]
   * @return {[type]} []
  ###
  @getIntervalProgress = =>
    return @frame / (@config.animation.fps - 1)

  ###*
   * [hide]
   * @return {[type]} []
  ###
  @hide = ($event) =>
    $event?.preventDefault()
    $event?.stopPropagation()

    @root?.removeClass?('guided-tour')

    @resetState()
    @deregisterBackButtonAction()
    $interval.cancel(@drawLoop)
    @clearCanvas('0,0,0', 0)
    GuidedTour.setPassed()
    $timeout(=>
      @shown = false
      @step = undefined
      @setText('')
      @scaleTextOut()
      @stopDrawLoop()
      #@showText(0)
    )
    window.removeEventListener('orientationchange', @setDimensions)
    return

  ###*
   *      _________    _   ___    _____   _____    __  ____________________    ____________  __
   *     / ____/   |  / | / / |  / /   | / ___/   / / / /_  __/  _/_  __/ /   /  _/_  __/\ \/ /
   *   / /   / /| | /  |/ /| | / / /| | \__ \   / / / / / /  / /  / / / /    / /  / /    \  /
   *  / /___/ ___ |/ /|  / | |/ / ___ |___/ /  / /_/ / / / _/ /  / / / /____/ /  / /     / /
   *  \____/_/  |_/_/ |_/  |___/_/  |_/____/   \____/ /_/ /___/ /_/ /_____/___/ /_/     /_/
  ###

  ###*
   * [getRGBAColor]
   * @param  {[type]} color   []
   * @param  {[type]} opacity []
   * @return {[type]}         []
  ###
  @getRGBAColor = (color, opacity) ->
    if !opacity?
      return 'rgba(' + color + ', 1)'
    return 'rgba(' + color + ',' + opacity + ')'

  ###*
   * [clearCanvas]
   * @return {[type]} []
  ###
  @clearCanvas = (color = @config.backdrop.color, opacity = @config.backdrop.opacity) =>
    @ctx.canvas.width  = @width
    @ctx.canvas.height = @height

    @ctx.clearRect(0, 0, @ctx.canvas.width, @ctx.canvas.height)

    @ctx.fillStyle = @getRGBAColor(color, opacity)
    @ctx.fillRect(0, 0, @width, @height)
    return

  ###*
   * [drawArrowHead]
   * @param  {[type]} arrow   []
   * @param  {[type]} start []
   * @param  {[type]} stepEnd   []
   * @param  {[type]} alpha   []
   * @return {[type]}         []
  ###
  @drawArrowHead = (arrow, start, stepEnd, opacity, control, bezierEnd, bezierControl) =>
    sizex = 18
    sizey = 18

    angle = @findBezierAngle(opacity, start, stepEnd, control, bezierEnd, bezierControl)
    angle += Math.PI / 2
    hx = sizex / 2
    hy = sizey / 2
    @ctx.translate((stepEnd.x),(stepEnd.y))
    @ctx.rotate(angle)
    @ctx.translate(-hx, -hy)
    @ctx.beginPath()
    @ctx.moveTo(0,0)
    @ctx.lineTo(0,1*sizey)
    @ctx.lineTo(1*sizex,1*hy)
    @ctx.closePath()
    @ctx.fill()
    @ctx.setTransform(1, 0, 0, 1, 0, 0)
    return

  ###*
   * [drawEllipse]
   * @param  {[type]} x   []
   * @param  {[type]} y   []
   * @param  {[type]} w   []
   * @param  {[type]} h   []
   * @param  {[type]} r=0 []
   * @param  {[type]} a   []
   * @return {[type]}     []
  ###
  @drawEllipse = (x, y, w, h, r=0, a) =>
    x -= r / 2
    y -= r / 2
    w += r
    h += r

    ox = (w / 2) * @KAPPA
    oy = (h / 2) * @KAPPA
    xe = x + w
    ye = y + h
    xm = x + w / 2
    ym = y + h / 2

    @ctx.beginPath()
    @ctx.moveTo(x, ym)
    @ctx.bezierCurveTo(x, ym - oy, xm - ox, y, xm, y)
    @ctx.bezierCurveTo(xm + ox, y, xe, ym - oy, xe, ym)
    @ctx.bezierCurveTo(xe, ym + oy, xm + ox, ye, xm, ye)
    @ctx.bezierCurveTo(xm - ox, ye, x, ym + oy, x, ym)
    @ctx.closePath()
    @ctx.fill()
    @ctx.lineJoin = @ctx.lineCap = 'round'
    @ctx.shadowBlur = 20
    @ctx.lineWidth = 0
    @ctx.stroke()

  ###*
   * [drawEllipseByCenter]
   * @param  {[type]} cx    []
   * @param  {[type]} cy    []
   * @param  {[type]} w     []
   * @param  {[type]} h     []
   * @param  {[type]} dummy []
   * @param  {[type]} a     =             10 []
   * @return {[type]}       []
  ###
  @drawEllipseByCenter = (cx, cy, w, h, dummy, a = 10) =>
    @drawEllipse(cx - w/2.0, cy - h/2.0, w, h, dummy, a)
    return

  ###*
   * [drawRoundedRectangle]
   * @param  {[type]} x []
   * @param  {[type]} y []
   * @param  {[type]} w []
   * @param  {[type]} h []
   * @param  {[type]} r []
   * @param  {[type]} a =             10 []
   * @return {[type]}   []
  ###
  @drawRoundedRectangle = (x, y, w, h, r, a = 10) =>
    if w < 2 * r
      r = w / 2
    if h < 2 * r
      r = h / 2
    @ctx.beginPath()
    @ctx.moveTo x + r, y
    @ctx.arcTo x + w, y, x + w, y + h, r
    @ctx.arcTo x + w, y + h, x, y + h, r
    @ctx.arcTo x, y + h, x, y, r
    @ctx.arcTo x, y, x + w, y, r
    @ctx.closePath()
    @ctx.fill()
    #@ctx.globalCompositeOperation="source-over"
    @ctx.lineJoin = @ctx.lineCap = 'round'
    @ctx.shadowBlur = 20
    @ctx.lineWidth = 0
    @ctx.stroke()
    return

  ###*
   * [getQuadraticBezierPoint]
   * @param  {[type]} step    []
   * @param  {[type]} anchor1 []
   * @param  {[type]} anchor2 []
   * @param  {[type]} control []
   * @return {[type]}         []
  ###
  @getQuadraticBezierPoint = (step, anchor1, anchor2, control) ->
    x = (1 - step) * (1 - step) * anchor1.x + 2 * (1 - step) * step * control.x + step * step * anchor2.x
    y = (1 - step) * (1 - step) * anchor1.y + 2 * (1 - step) * step * control.y + step * step * anchor2.y
    return x : x, y : y

  ###*
   * [prepareBrush]
   * @param  {[type]} strokeStyle              []
   * @param  {[type]} globalCompositeOperation []
   * @param  {[type]} fillStyle=strokeStyle    []
   * @return {[type]}                          []
  ###
  @prepareBrush = (strokeStyle, globalCompositeOperation, fillStyle=strokeStyle) =>
    @ctx.strokeStyle = @strokeStyle   = strokeStyle
    @ctx.fillStyle   = @fillStyle     = fillStyle
    @ctx.globalCompositeOperation = @globalCompositeOperation = globalCompositeOperation || @globalCompositeOperation
    return

  ###*
   * [setText]
   * @param {[type]} text []
  ###
  @setText = (text) =>
    if @text == $filter('translate')(text)
      return

    return $timeout(=>
      @text = $filter('translate')(text)
      $scope.getSelf().find('.content').html(@text)
    , 0, false)

  ###*
   * [clearText]
   * @return {[type]} []
  ###
  @clearText = =>
    @animate.fadeOut = true

    @timeouts.push $timeout(=>
      @setText('')
    , 250, false)

    @timeouts.push $timeout(=>
      @animate.fadeOut = false
    , 500, false)


  ###*
   * [checkForIntersection]
   * @param  {[type]} element ]
   * @return {[type]}         ]
  ###
  @checkForIntersection = (element) ->
    $hint           = $scope.getSelf().find('#hint')
    hintTop         = $hint[0].getBoundingClientRect().top
    hintBottom      = hintTop + $hint[0].offsetHeight

    cHighlightTop    = element[0].getBoundingClientRect().top
    cHighlightBottom = element[0].getBoundingClientRect().top + element[0].offsetHeight

    if hintTop >= cHighlightTop && hintTop <= cHighlightBottom || hintBottom >= cHighlightTop && hintBottom <= cHighlightBottom
      return true

    return false

  ###*
   * [avoidIntersection]
   * @return {[type]} []
  ###
  @avoidIntersection = =>
    for h in @currentHighlight
      if @checkForIntersection(h.el) then return true

    return false

  ###*
   * [drawArrowToElement]
   * @param  {[type]} element                  []
   * @param  {[type]} alpha=@cc.animationSteps []
   * @return {[type]}                          []
  ###
  @drawArrowToElement = (element, opacity, fade) =>
    $hint = $scope.getSelf().find('#hint')

    # prepare the start and end point for the arrow
    start =
      x: @width / 2
    bezierEnd =
      x: element[0].getBoundingClientRect().left + element[0].offsetWidth / 2

    ###
    if bezierEnd.x > start.x
      start.x += 24
    else
      start.x -= 24
    ###

    # determine if arrow has to end at the upper or lower end
    if $hint[0].getBoundingClientRect().top > element[0].getBoundingClientRect().top + element[0].offsetHeight
      start.y = $hint[0].getBoundingClientRect().top - 12
      bezierEnd.y = element[0].getBoundingClientRect().bottom + 24
    else
      start.y = $hint[0].getBoundingClientRect().bottom + 12
      bezierEnd.y = element[0].getBoundingClientRect().top - 24

    # add the control point for the bezier curve
    bezierControl =
      x : bezierEnd.x
      y : start.y

    @prepareBrush(@getRGBAColor('255,255,255,1'), 'source-over')
    @drawEllipseByCenter(start.x, start.y, 12, 12)
    @drawEllipseByCenter(start.x, start.y, 6, 6)

    ###
    bezierControl =
      x : start.x
      y : bezierEnd.y
    ###

    if !fade
      # retrieve the end point for the current animation step
      stepEnd = @getQuadraticBezierPoint(opacity, start, bezierEnd, bezierControl)

      # retrieve the animation step control point
      stepControl =
        x : bezierEnd.x - (1 - opacity) * (bezierEnd.x - start.x)
        y : start.y

      ###
      stepControl =
        x : start.x
        y : bezierEnd.y - (1 - opacity) * (bezierEnd.y - start.y)
      ###

      # and draw!
      @prepareBrush(@getRGBAColor('255,255,255', opacity), 'source-over')
      @drawCubic(start, stepEnd, opacity, stepControl)
      @drawArrowHead({h : 12, w : 12}, start, stepEnd, opacity, stepControl, bezierEnd, bezierControl)
    else
      fadeOpacity = 1 - 1 / @config.animation.fps

      stepEnd = @getQuadraticBezierPoint(fadeOpacity, start, bezierEnd, bezierControl)
      # retrieve the animation step control point
      stepControl =
        x : bezierEnd.x - (1 - fadeOpacity) * (bezierEnd.x - start.x)
        y : start.y

      # and draw!
      @prepareBrush(@getRGBAColor('255,255,255', opacity), 'source-over')
      @drawCubic(start, stepEnd, opacity, stepControl)
      @drawArrowHead({h : 12, w : 12}, start, stepEnd, fadeOpacity, stepControl, bezierEnd, bezierControl)

    return

  @drawDebugCircle = (point, color='white', radius=12) =>
    if !@debug
      return

    @prepareBrush(color, 'source-over')
    @drawEllipseByCenter(point.x, point.y, radius, radius)
    @ctx.stroke()
    return

  ###*
   * [drawCubic]
   * @param  {[type]} startPt []
   * @param  {[type]} endPt   []
   * @param  {[type]} alpha   []
   * @return {[type]}         []
  ###
  @drawCubic = (startPt, endPt, alpha, control) =>
    @drawDebugCircle(control, 'red', 12)

    @ctx.moveTo(startPt.x, startPt.y)
    @ctx.lineWidth = 4
    @ctx.quadraticCurveTo(control.x, control.y, endPt.x, endPt.y)
    @ctx.stroke()
    return

  ###*
   * [drawLine]
   * @param  {[type]} startPt []
   * @param  {[type]} endPt   []
   * @param  {[type]} alpha   []
   * @return {[type]}         []
  ###
  @drawLine = (startPt, endPt, alpha) =>
    @ctx.moveTo(startPt.x, startPt.y)
    @ctx.lineWidth = 3
    @ctx.lineTo(endPt.x, endPt.y)
    @ctx.stroke()
    return

  ###*
   * [drawCalculatedBezier - helper]
   * @param  {[type]} start   []
   * @param  {[type]} end     []
   * @param  {[type]} control []
   * @return {[type]}         []
  ###
  @drawCalculatedBezier = (start, end, control) =>
    if !@debug
      return

    steps = 10
    i = 0
    while i < steps
      step = i / 10
      point = @getQuadraticBezierPoint(step, start, end, control)
      @drawDebugCircle(point, 'yellow', 8)
      i++

  ###*
   * [findBezierAngle]
   * @param  {[type]} value         []
   * @param  {[type]} anchor1       []
   * @param  {[type]} anchor2       []
   * @param  {[type]} control       []
   * @param  {[type]} bezierEnd     []
   * @param  {[type]} bezierControl []
   * @return {[type]}               []
  ###
  @findBezierAngle = (value, anchor1, anchor2, control, bezierEnd, bezierControl) =>
    na1 = @getQuadraticBezierPoint(value - 0.1, anchor1, bezierEnd, bezierControl)
    na2 = @getQuadraticBezierPoint(value + 0.1, anchor1, bezierEnd, bezierControl)

    @drawDebugCircle(na1, 'orange', 8)
    @drawDebugCircle(na2, 'orange', 8)

    a = Math.abs(na2.y - na1.y)
    b = Math.abs(na2.x - na1.x)
    c = Math.sqrt(a*a + b*b)

    if na1.x > na2.x && na1.y <= na2.y || na1.y > na2.y && na1.x <= na2.x
      alpha = Math.PI * 2 - Math.atan2(a, b) + Math.PI / 2
    else
      alpha = Math.atan2(a, b) + Math.PI * 1.5

    if na1.y > na2.y
      alpha += Math.PI

    return alpha

  @init()

  return
)