angular.module("Flexikon.directives").directive('dcmediaviewer', (PicturesApi, $ionicGesture, $timeout, $ionicSlideBoxDelegate, NavBar, PictureSlider) ->
	restrict: 'E'
	scope: true
	templateUrl: 'templates/DcMediaViewerTemplate.html'
	link: (scope, element, attrs) ->
		
		scope.pictures =
			ids : attrs.items.replace(/img_/g, '').split(',')

		scope.current =
			index : 0

		scope.ui =
			width     : window.innerWidth
			widththird: (window.innerWidth) / 3
			loading   : true
			error     : false
			noPictures: false
			maxHeight : 200

		scope.slider =
			position : 0

		PicturesApi.getPicturesForMediaViewer(scope.pictures.ids)
		.success((result) ->
			if result.length == 0
				scope.pictures.results = []
				scope.ui.nopictures = true
				return	
			scope.ui.nopictures = false
			scope.pictures.results = result._items.pictures_photo_app
		).error((error) ->
			console.warn error
			scope.ui.error = true
		).finally(->
			scope.ui.loading = false
		)

		scope.onSliderHide = (index) ->
			scope.current.index = index
			scope.setScrollPosition()

		scope.getSliderStyle = ->
			return {
				'-webkit-transform' : 'translate3d(-' + scope.slider.position + 'px, 0, 0)'
				'transform' : 'translate3d(-' + scope.slider.position + 'px, 0, 0)'
			}

		scope.clicked = ($event, $index) ->
			if scope.current.index == $index
				scope.modalapi.show(scope.pictures.results, '', scope.current.index)
				return
				
			scope.current.index = $index
			scope.slider.position = scope.current.index * $event.target.width
			return

		scope.increaseIndex = (apply=false) ->
			if scope.current.index < scope.pictures.results.length - 1
				scope.current.index++
				scope.setScrollPosition(apply)

		scope.decreaseIndex = (apply=false) ->
			if scope.current.index > 0
				scope.current.index--
				scope.setScrollPosition(apply)

		scope.goToIndex = (index) ->
			scope.current.index = index
			scope.setScrollPosition()

		scope.close = ->
			scope.ui.bigPicture = false

		scope.getItemDimension = ->
			dim = scope.ui.width / 3
			margin = 0

			if scope.ui.width / 3 > scope.ui.maxHeight
				margin = (dim - scope.ui.maxHeight + 20) / 2
				
				dim = scope.ui.maxHeight - 20

			return {
				'width' : dim + 'px'
				'height' : dim + 'px'
				'margin' : margin + 'px'
			}

		scope.setScrollPosition = (apply=false) ->
			scope.slider.position = scope.current.index * scope.ui.width / 3
			if apply then scope.$$phase || scope.$digest()

		scope.refresh = ->
			scope.setScreenWidth()			
			scope.$$phase || scope.$digest(->
				scope.getItemDimension()
				scope.setScrollPosition()
			)

			width = window.innerWidth

			$timeout(->
				if width == window.innerWidth
					return 
					
				scope.setScreenWidth()			
				scope.$$phase || scope.$digest(->
					scope.getItemDimension()
					scope.setScrollPosition()
					$timeout(->
						scope.setScrollPosition()
					)					
				)
			, 1000)

		window.addEventListener('orientationchange', scope.refresh)

		scope.setScreenWidth = ->
			scope.ui.width = window.innerWidth - 12
			scope.ui.widththird = (window.innerWidth - 12) / 3

		###*
		 *
		 *	Gesture handling
		 * 
		###

		###*
		 * [onDragLeft - drag left gesture]
		 * @return
		###
		scope.onDragLeft = ->
			event.preventDefault()
			event.stopPropagation()
			event.gesture.stopDetect()
			event.gesture.preventDefault()
			event.gesture.stopPropagation()
			if scope.current.index < scope.pictures.results.length - 1
				scope.increaseIndex(true)	
		scope.dragLeftAction = $ionicGesture.on('dragleft', scope.onDragLeft, element)

		###*
		 * [onDragRight - drag right gesture]
		 * @return
		###
		scope.onDragRight = ->
			event.preventDefault()
			event.stopPropagation()
			event.gesture.stopDetect()
			event.gesture.preventDefault()
			event.gesture.stopPropagation()
			if scope.current.index > 0
				scope.decreaseIndex(true)
		scope.dragRightAction = $ionicGesture.on('dragright', scope.onDragRight, element)

		###*
		 * [remove drag gesture handlers on destroy]
		###
		scope.$on('$destroy', ->
			window.removeEventListener('orientationchange', scope.refresh)
			$ionicGesture.off(scope.dragLeftAction, 'dragleft', scope.onDragLeft)
			$ionicGesture.off(scope.dragRightAction, 'dragright', scope.onDragRight)
		)

		PictureSlider.init(scope)

)