# DcHighlight service

angular.module("Flexikon.services").factory("DcHighlight",  ->

  @amount = 0
  @current = 0
  @focused = -1

  ###*
   * [highlightWord]
   * @param  {[type]} root []
   * @param  {[type]} word []
   * @return {[type]}      []
  ###
  @highlightWord = (root, word) =>
    @removeHighlights(root)

    if !word? || word.length == 0
      return

    if !root?
      return

    result = @textNodesUnder(root)
    @highlightIntercepted(result, word)
    return @amount

  ###*
   * [textNodesUnder]
   * @param  {[type]} root []
   * @return {[type]}      []
  ###
  @textNodesUnder = (root) ->
    walk = document.createTreeWalker(root, NodeFilter.SHOW_TEXT, null, false)
    text = []
    node = undefined

    while(node = walk.nextNode())
      text.push(node)

    return text

  ###*
   * [removeHighlights]
   * @param  {[type]} root []
   * @return {[type]}      []
  ###
  @removeHighlights = (root) =>
    @amount = 0
    @current = 0
    @focused = -1

    highlights = Array.apply(null, root.querySelectorAll('.highlighted')).reverse()

    for highlight in highlights
      wrapper = highlight.parentElement

      if highlight.innerHTML.trim().length == 0
        wrapper.removeChild(highlight)
      else
        plainText = document.createTextNode(highlight.innerHTML)
        wrapper.replaceChild(plainText, highlight)

    root.normalize()

    return @amount

  ###*
   * [highlightIntercepted]
   * @param  {[type]} nodes []
   * @param  {[type]} word  []
   * @return {[type]}       []
  ###
  @highlightIntercepted = (nodes, word) =>

    plainTextNodes = nodes.map((node, index) ->
      return '#' + index + '#' + node.nodeValue
    ).join('')

    splittedWord  = word.split(/(?!^)/gmi)
    wildcardWord  = '(#\\d+#)[^#]*' + splittedWord.join('(#\\d+#)* *')
    wildcardRegex = new RegExp(wildcardWord ,"gmi")

    matches = (plainTextNodes.match(wildcardRegex))

    if !matches?
      return

    matches.forEach((match) =>

      nodeIndexes = match.match(/#(\d+)#/gmi)

      if !nodeIndexes?
        return

      if nodeIndexes.length > 1
        nodeIndexes.forEach((nodeIndex) =>
          nodeIndex = parseInt(nodeIndex.replace(/#/gmi, ''))

          leftNode      = nodes[nodeIndex-1]
          lastWord      = splittedWord.join('?') + '?$'
          lastWordRegex = new RegExp(lastWord ,"gmi")
          lastMatch     = leftNode.nodeValue.match(lastWord)
          lastLength    = lastMatch[0].length
          lastIndex     = lastMatch.index

          if !lastMatch? || lastMatch.length == 0 || lastMatch[0].length == 0
            return
          @highlight(leftNode, word.substr(lastIndex, lastLength), lastIndex)

          rightNode      = nodes[nodeIndex]
          firstWord      = '^' + splittedWord.join('?') + '?'
          firstWordRegex = new RegExp(firstWord ,"gmi")
          firstMatch     = rightNode.nodeValue.match(firstWord)
          firstLength    = firstMatch[0].length
          firstIndex     = firstMatch.index

          @highlight(rightNode, word.substr(firstIndex, firstLength), firstIndex, false)
        )
      else
        nodeIndex = parseInt(nodeIndexes[0].replace(/#/gmi, ''))

        node = nodes[nodeIndex]

        @highlight(node, word, node.nodeValue.toLowerCase().indexOf(word.toLowerCase()))

    )

  ###*
   * [highlight ]
   * @param  {[type]} node   []
   * @param  {[type]} length []
   * @param  {[type]} i      []
   * @return {[type]}        []
  ###
  @highlight = (node, word, i, count=true) =>

    parent = node.parentNode
    if parent.className.indexOf('highlighted') >= 0
      return

    if word.length == 0 || i < 0
      return

    if count
      @amount++

    span           = document.createElement('span')
    span.className = 'highlighted'
    span.id        = 'dc-highlight-' + (@amount-1)

    if i == 0 && i + word.length == node.nodeValue.length
      # only word; replace whole node by span
      parent.replaceChild(span, node)
      span.appendChild(node)
      return
    else if i == 0
      # word at the beginning; split after
      after = node.splitText(i + word.length)
      parent.replaceChild(span, node)
      span.appendChild(node)
    else if i + word.length == node.nodeValue.length
      # word at the end
      highlighted = node.splitText(i)
      parent.replaceChild(span, highlighted)
      span.appendChild(highlighted)
      return
    else
      # word somewhere in the middle
      after          = node.splitText(i + word.length)
      highlighted    = node.splitText(i)
      span.appendChild(highlighted)
      after.parentNode.insertBefore(span,after)

    i = after.nodeValue.toLowerCase().indexOf(word.toLowerCase())

    if i >= 0
      @highlight(after, word, i)

    return

  ###*
   * [unfocus]
   * @return {[type]} []
  ###
  @unfocus = =>
    if @focused >= 0
      focusedNodes = document.querySelectorAll('#dc-highlight-' + @focused)

      for node in focusedNodes
        if node?
          node.className = node.className.replace(' focused', '')

  ###*
   * [focus]
   * @param  {[type]} count []
   * @return {[type]}       []
  ###
  @focus = (count) =>
    if @focused == count
      return

    @unfocus()

    @focused = count
    focusedNodes = document.querySelectorAll('#dc-highlight-' + count)
    for node in focusedNodes
      if node.className.indexOf(' focused') < 0
        node.className += ' focused'

  ###*
   * [isHighlightNode]
   * @param  {[type]}  node []
   * @return {Boolean}      []
  ###
  @isHighlightNode = (node) ->
    return node?.tagName?.toLowerCase() == 'span' && node.className.indexOf('highlighted') >= 0

  return @
)