DcPouchDB = ((pouchDB) ->

  ###*
   * [prepareDBName]
   * @param  {[type]} database []
   * @return {[type]}          []
  ###
  prepareDBName: (database) ->
    @logging = true

    prefix = 'help_patient_'

    ua = navigator.userAgent

    if (ua.indexOf("Android") >= 0)
      version = parseFloat(ua.slice(ua.indexOf("Android") + 8))
      if (version <= 4.3)
        prefix = 'websql://' + prefix

    @dbname = prefix + database

    ionic.Platform.ready =>
      #@logger = @log.getInstance('PouchDB - ' + @dbname, 'color: #373')
      if @logger? && @logging then @logger?.debug('initialized |')

    return @dbname

  ###*
   * [getAll]
   * @param  {[type]} limit []
   * @return {[type]}       []
  ###
  getAll: (limit = 0)->
    if @logging
      if @logger? then @logger?.debug('getAll')
    self = @

    options = {}
    if limit > 0 then options.limit = limit

    @db.allDocs(options).then((results) ->
      result = []

      for doc in results.rows
        result.push self.db.get(doc.id)

      return self.q.all(result)
    )

  ###*
   * [get]
   * @param  {[type]} id []
   * @return {[type]}    []
  ###
  get: (id) ->
    if @logging
      if @logger? then @logger?.debug('get | %s', id)
    return @db.get(id + '')

  ###*
   * [insert]
   * @param  {[type]} id='-1'  []
   * @param  {[type]} document []
   * @return {[type]}          []
  ###
  insert: (id='-1', document) ->
    if @logging
      if @logger? then @logger?.debug('insert | id : %s | value : %o', id, document)
    return @db.post
      _id : id + ''
      article : document

  ###*
   * [update]
   * @param  {[type]} doc []
   * @return {[type]}     []
  ###
  update: (doc) ->
    if @logging
      if @logger? then @logger?.debug('update | article : %o', doc)
    return @db.put doc

  ###*
   * [insertOrUpdate]
   * @param  {[type]} id    []
   * @param  {[type]} value []
   * @return {[type]}       []
  ###
  insertOrUpdate: (id, value) ->
    if @logging
      if @logger? then @logger?.debug('insert or update | id : %s | value : %o', id, value)
    return @db.get(id + '')
    .then((doc) =>
      doc.article = value
      return @update(doc)
    ).catch((error) =>
      return @insert(id, value)
    )

  ###*
   * [exists]
   * @param  {[type]} id []
   * @return {[type]}    []
  ###
  exists: (id) ->
    if @logging
      if @logger? then @logger?.debug('exists | %s', id)
    self = @
    return @q((resolve, reject) ->
      self.get(id + '').then(->
        resolve(true)
      ).catch(->
        resolve(false)
      )
    )

  ###*
   * [delete]
   * @param  {[type]} doc []
   * @return {[type]}     []
  ###
  delete: (doc) ->
    if @logging
      if @logger? then @logger?.debug('delete | %o', doc)
    return @db.remove(doc)

  ###*
   * [clear]
   * @return {[type]} []
  ###
  clear: ->
    if @logging
      if @logger? then @logger?.debug('clear')

    self = @

    @getAll().then((results) ->
      result = []

      for doc in results
        result.push self.delete(doc)

      return self.q.all(result)
    )
)()