angular.module("Flexikon.services").factory("Login", ($http, $rootScope, Api, LocalStorage, authService, Load, GlobalSettings, Favorites, UserSave, LastSeen, LoginDB) ->

	###
	#	Login Constants
	###
	url 						= 'https://login.doccheck.com/'
	tokenDomain 		= 'service/oauth/access_token/'

	loginId 				= '2000000008360'
	clientSecret 		= 'AhChoo!Opqlkt2$h!'
	strDcLanguage		= GlobalSettings.Language()
	grantType 			= 'authorization_code'
	intialized			= false
	accessToken 		= ''

	###
	#	Initialiaze on app restart
	###
	init : ->
		if !@initialized 
			console.debug '#=-=Initializing Login...=-=#=-=#=-=#'

			$http.defaults.headers.post['Content-Type'] = false
			$http.defaults.headers.post['Allow-Control-Allow-Origin'] = '*'
			$http.defaults.headers.common['Accept'] = 'application/json'

			LoginDB.get('dc_flexikon_access_token')
			.then((token) =>
					$http.defaults.headers.common['Authorization'] = token.article
					@accessToken = token.article
					console.debug '#=-=Token restore completed =-=#=-=#=-=#'
					console.debug '# Restored oAuth token from local database'
					console.debug '#'
					console.debug '# Current authorization header: '
					console.debug '# ' + $http.defaults.headers.common.Authorization
					console.debug '#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#\n\n'
					authService.loginConfirmed()
			).catch(=>
				@accessToken = ''
				console.debug '#=-=Token restore failed =-=#=-=#=-=#'
				console.debug '# No token to restore'
				console.debug '#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#\n\n'
			)

			strDcLanguage	= GlobalSettings.Language()

	###
	# Collect token
	###
	collectToken : (code) ->
		return $http({
			url: url + tokenDomain
			method: "POST"
			cache: false
			params: 
				client_id : loginId
				client_secret : clientSecret
				grant_type : grantType
				code : code
		})

	###
	# Login
	###
	sendLoginRequest : (username, password, successCallback, errorCallback) ->
		self = @

		$http({
			url: url 
			method: "POST"
			cache: false
			params: {
				login_id 			: loginId
				strDcLanguage : strDcLanguage
				language 			: strDcLanguage
				username 			: username
				password			: password
			}
		}).success((data, status, headers, config) ->

			if data.boolSuccess

				LocalStorage.set('dc_flexikon_user_id', data.strUserId)				

				UserSave.get(LocalStorage.get('dc_flexikon_user_id')).then((data) ->
					userData = JSON.parse(data.article)
					for ls in userData.lastSeen 
						LastSeen.insert(ls._id, ls.article)
					for fav in userData.favorites
						Favorites.insert(fav._id, fav.article)
				, (error) ->
					console.debug 'Couldn\'t fetch favorites for ' + LocalStorage.get('dc_flexikon_user_id')
				)

				self.collectToken(data.arrUrlParams.code).success((data, status, headers, config) ->

					if !data.access_token
						
						self.errorFunction()

						errorCallback?(data, status, headers, config)
						authService.loginCancelled()
						return

					token = data.token_type + " " + data.access_token

					console.debug '#=-=New access token:=-=#=-=#=-=#=-=#'
					console.debug '# ' + token
					console.debug '#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#\n\n'

					self.saveToken(token)
					self.setDefaultHeader(token)
				
					successCallback?(data, status, headers, config)
					authService.loginConfirmed()

				).error((data, status, headers, config) ->
					self.errorFunction()
					errorCallback?(data, status, headers, config)
					authService.loginCancelled()
				)

			else
				self.errorFunction()
				errorCallback?(data, status, headers, config)
				authService.loginCancelled()

		).error((data, status, headers, config) ->

			self.errorFunction()

			errorCallback?(data, status, headers, config)
			authService.loginCancelled()

		)

	errorFunction : ->
		@deleteToken()
		@unsetDefaultHeader()

	###
	#	Logout
	###
	signOut : ->		
		@deleteToken()
		@unsetDefaultHeader()

		always = ->
			Favorites.clear()
			LastSeen.clear()
			
			LocalStorage.unset('pushToken')
			LocalStorage.set('last_dc_flexikon_user_id', LocalStorage.get('dc_flexikon_user_id'))
			LocalStorage.unset('dc_flexikon_user_id')

			$rootScope.$broadcast 'event:auth-logout'
			return

		Favorites.getAll().then((favData)-> 
			
			LastSeen.getAll().then((lsData) ->
				
				saveObject = 
					lastSeen : lsData
					favorites : favData

				UserSave.get(LocalStorage.get('dc_flexikon_user_id')).then((doc) ->				
					doc.article = JSON.stringify(saveObject)
					UserSave.update(doc)
					always()
				, (error) ->
					UserSave.insert(LocalStorage.get('dc_flexikon_user_id'), JSON.stringify(saveObject))
					always()
				)

			)

		)

	###
	# Determine if a token is present and if it's still valid
	###
	isAuthenticated : ->
		return (
			@accessToken? and @accessToken.length > 0 and
			#LocalStorage.isSet('dc_flexikon_access_token') and
			parseInt(LocalStorage.get('dc_flexikon_token_expires_at')) > (Date.now())
		)

	###
	#	Save token + expiry
	###
	saveToken : (token) ->
		console.debug token

		loginTime = new Date().getTime()
		expiresIn = 1000 * 60 * 60 * 24* 31 * 6

		@accessToken = token
		LoginDB.get('dc_flexikon_access_token')
		.then((doc) =>
			doc.article = token
			LoginDB.update(token)
		).catch(=>
			LoginDB.insert('dc_flexikon_access_token', token)
		)

		#LocalStorage.set('dc_flexikon_access_token', token)
		LocalStorage.set('dc_flexikon_token_expires_at', loginTime + expiresIn)

	###
	#	
	###
	deleteToken : ->
		@accessToken = ''
		LoginDB.get('dc_flexikon_access_token')
		.then((doc) =>
			LoginDB.delete(doc)
		)
		#LocalStorage.unset('dc_flexikon_access_token')

	setDefaultHeader : (token) ->
		$http.defaults.headers.common.Authorization = token

	unsetDefaultHeader : ->
		$http.defaults.headers.common.Authorization = ''

	getAccessToken : ->
		return @accessToken

)