angular.module("Flexikon.services").factory("WikiHTML", (GlobalSettings, $timeout) ->

  self = @
  self.lists = []
  self.placeholders = []

  ###*
   * [parseTable - parse a complete Wiki syntax table to html and wrap it with a dc-table directive for styling and scrolling purposes]
   * @param  {[String]} tableinner [content of the table]
   * @return {[String]}            [parsed table]
  ###
  self.parseTable = (tableinner) ->
    tableinner = tableinner.replace(/\n( |\t)*/gmi, '\n').replace(/\n *\n/gmi, '\n')

    tablerows = tableinner.trim().split(/(?:^|\n)\|-(.*)(?:\n)?/g) # TODO Use styles if available
    table =
      caption : null
      header  : []
      rows    : []

    ###
    if tablerows[0].match(/\n!/gmi)?.length > 0
      heads = tablesrows.unshift()
      heads = heads.replace(/\n!/gmi, '\\n!!').split(/\n!/gmi)
      tablerows.unshift.apply(heads)
    ###

    for row in tablerows
      if      row.indexOf('|+ ') == 0 then table.caption = row.substr(('|+ ').length)
      else if row.indexOf('! ')  == 0 then table.header.push.apply(table.header, row.replace(/^! *\|? */, '').split(/\n! |\ !! | \|\| /g))
      else if row.indexOf('!')  == 0
        headerrows = row.replace(/^! *\|?!? */gmi, '').split(/\n! *|\ !! | \|\| /gmi)

        if headerrows.length == 1
          headerrows = headerrows[0].split(/\n/gmi)

        table.header.push.apply(table.header, headerrows)
      else if row.indexOf('|')   == 0 then table.rows.push(row.replace(/^\| /, '').split(/\n\||\|\|/g))

    # prepare the table wrapper
    tableHtml = "<dc-table><ion-scroll has-bouncing='false' direction='x'><table>"
    if table.caption? then tableHtml += "<caption>#{table.caption}</caption>"

    # parse the table header
    if (table.header.length == 0 && table.rows.length == 1 ||
        table.header.length == 1 && table.rows.length == 0)
      tableHtml = tableHtml.replace('<table', '<table class="no-stack single-row"')
    if table.header.length > 0
      tableHtml += "<tr>"
      for header in table.header
        header = header.split(/\n\| /g)
        th = header.splice(0, 1)
        tableHtml += "<th>" + th + "</th>"
        for columns in header
          hasColumns = true
          tableHtml = tableHtml.replace('<table', '<table class="no-stack"')
          tableHtml += "<td>" + columns + "</td>"
        if hasColumns then tableHtml += '</tr><tr>'
      tableHtml += ""
      tableHtml += '</tr>'
    
    # parse table rows
    if table.rows.length > 0
      for row in table.rows
        tableHtml += "<tr>"
        for col in row
          col = col.split('|')
          if col.length == 2
            tableHtml += "<td " + col[0] + ">" + col[1] + "</td>"
          else if col.length == 1
            tableHtml += "<td>" + col[0] + "</td>"
        tableHtml += "</tr>"
        
    # close the whole stuff
    tableHtml += "</table></ion-scroll></dc-table>\n"

    # deprecated stack tables
    if tableinner.indexOf('<dctv') > 0
      tableHtml = tableHtml.replace('<table', '<table class="no-stack"')

    return tableHtml

  ###*
   * [parseCompleteList - parse a complete list (might consist of multiple nested ols and uls )]
   * @param  {[String]} match [the list match from the according regex]
   * @return {[String]}       [returns the list HTML]
  ###
  self.parseCompleteList = (match) ->

    listHTML = ''

    ###*
     * [tagForChar - helper for parsing lists]
     * @param  {[char]} c [Wiki syntax character]
     * @return {[String]} [the according list tag name]
    ###
    tagForChar = (c) ->
      if c == '*' then return 'ul'
      if c == '#' then return 'ol'
      return 'UNDEFINED'

    ###*
     * [findFirstUnmatched - find the position of the first indicator not matching the current]
     * @param  {[String]} indicator [current indicator sequnce]
     * @return {[int]}              [position of the first unmatched]
    ###
    findFirstUnmatched = (indicator) ->
      pos = -1
      for i in [indicator.length-1..0]
        if indicator[i] != self.lists[i]
          pos = i
      return pos

    ###*
     * [closeTagsToLevel - helper for closing all currently open tags down to a certain level]
     * @param  {[int]} l [level]
     * @return
    ###
    closeTagsToLevel = (l) ->
      while self.lists.length > l
        listHTML += '</li></' + tagForChar(self.lists.pop()) + '>'

    ###*
     * [openTagsFromLevel - helper for opening tags for lists up to a certain level]
     * @param  {[int]} l          [level]
     * @param  {[char]} indicator [Wiki syntax list indicator]
     * @return {[type]}
    ###
    openTagsFromLevel = (l, indicator) ->
      for i in [l..indicator.length-1]
        listHTML += '\n<' + tagForChar(indicator[i]) + '><li>'
        self.lists.push indicator[i]
        
    rows = match.split('\n')
    for r in rows
      if r.indexOf? && (r.indexOf('*') == 0 || r.indexOf('#') == 0)
        match     = r.match(/^([\*\#]+)(.*)$/)
        indicator = match[1]
        content   = match[2]

        if indicator.length < self.lists.length
          closeTagsToLevel(indicator.length)

        unmatched = findFirstUnmatched(indicator)
        if unmatched >= 0
          closeTagsToLevel(unmatched)
          #if self.lists.length > 0 then listHTML += '<li>'
          openTagsFromLevel(unmatched, indicator)
        else
          listHTML += '</li><li>'

        listHTML += content

    closeTagsToLevel(0)

    listHTML += '\n'

    return listHTML

  rules :
    toWiki :
      blocks : [
        # dctable removal
        rex: /<dc-table>(.*?)<\/dc-table>/g,
        to : (match, s1) -> return s1
      ,
        # ol
        rex : /<ol>(.*?)<\/ol>/g,
        to  : (match, s1) -> return match.replace(/\<ol>/g, '\n').replace(/\<li>/g, '# ').replace(/\<\/li>/g, '\n').replace(/\<\/ol>/g, '')
      , # ul
        rex : /<ul>(.*?)<\/ul>/g,
        to  : (match, s1) -> return match.replace(/\<ul>/g, '\n').replace(/\<li>/g, '* ').replace(/\<\/li>/g, '\n').replace(/\<\/ul>/g, '')
      , # table wrap
        rex : /<table>(.*?)<\/table>/g,
        to  : (match, s1) -> return match.replace(/\<table>/g, '{|').replace(/\<\/table>/g, '\n|}\n')
      , # table wrap
        rex : /<tbody>(.*?)<\/tbody>/g,
        to  : (match, s1) -> return match.replace(/\<tbody>|<\/tbody>/g, '')
      , # table heads
        rex : /\<tr>(.*?)(<th>(.*)<\/th>)(.*?)<\/tr>/g
        to  : (match, s1, s2) -> return s2.replace(/(<tr>)|(<\/tr>)/g, '').replace(/<th>/g, '\n! \'\'\'').replace(/<\/th>/g, '\'\'\'')
      , # table rows
        rex : /\<tr>(.*?)(<td>(.*)<\/td>)(.*?)<\/tr>/g
        to  : (match, s1, s2) -> return '\n' + s2.replace(/(<tr>)/g, '\n').replace(/<\/tr>/g, '').replace(/<\/td>.*?<td>/g, '||').replace(/<td>/g, '\|-\n\| ').replace(/<\/td>/g, '')
      ]
      inlines : [
      # remove empty p
        rex : /<p>([ \t\f\v\xB6]*?)<\/p>/g,
        to  : (match, s1) -> return s1
      , # h1
        rex : /<h1>(.*?)<\/h1>/mgi
        to  : (match, s1) -> return '='+s1+'=\n'
      , # h2
        rex : /<h2>(.*?)<\/h2>/mgi
        to  : (match, s1) -> return '=='+s1+'==\n'
      , # h3
        rex : /<h3>(.*?)<\/h3>/mgi
        to  : (match, s1) -> return '==='+s1+'===\n'
      , # h4
        rex : /<h4>(.*?)<\/h4>/mgi
        to  : (match, s1) -> return '===='+s1+'====\n'
      , # h5
        rex : /<h5>(.*?)<\/h5>/mgi
        to  : (match, s1) -> return '====='+s1+'=====\n'
      , # bold
        rex : /(<b>|<strong>)(.*?)(<\/strong>|<\/b>)/mgi
        to  : (match, s1, s2) -> return '\'\'\''+s2+'\'\'\''
      , # italic
        rex : /(<em>|<i>)(.*?)(<\/em>|<\/i>)/mgi
        to  : (match, s1, s2) -> return '\'\''+s2+'\'\''
      ]

    ###
    # ToDo:
    # - FlexiAds - Aspirin
    # - DcTv - Herz
    ###

    toHTML :
      prioInlines : [
        rex : /<!--(.|\n)*-->/gmi
        to : (match) -> return ''
      ,
        # references tags
        rex : /<(?:.*?)[^\s-]\/>/g
        to  : (match, s1, s2) -> return match.replace(/\/>/g, ' />')
      ,
        # Infobox
        # {{
        # |
        #
        #rex : /\{\{((?:\n|.)*(?!\n\}\})(?:\n|.*))\}\}/gm
        rex : /\{\{Quiz((?:\n|.)*?)\}\}/gm
        to : (match) -> return ''
      ,
        # Infobox
        # {{
        # |
        #
        #rex : /\{\{((?:\n|.)*(?!\n\}\})(?:\n|.*))\}\}/gm
        rex : /\{\{((?:\n|.)*?)\}\}/gm
        to : (match) ->
          self.placeholders.push match
          return '<dc-parse-wiki wiki-text="~~PLACEHOLDER' + (self.placeholders.length - 1) + '~~"></dc-parse-wiki>'
      , # replace all < (prevent html interpretation except for... [see next rule])
        rex : /<(?:\/?)(?:(?:.|\n)*?)>/gm
        to : (match) ->
          return match.replace('<', '&lt;')
      , # the following tags can be used in a flexikon article
        rex : /&lt;(?:\/?)(?:b|bdi|del|i|ins|u|font|big|small|sub|sup|h1|h2|h3|h4|h5|h6|cite|code|em|s|strike|strong|tt|var|div|center|blockquote|ol|ul|dl|table|caption|pre|ruby|rb|rp|rt|rtc|p|span|abbr|dfn|kbd|samp|data|time|mark|br|wbr|hr|li|dt|dd|table|tr|td|th|div|blockquote|ol|ul|li|dl|dt|dd|font|big|small|sub|sup|var|kbd|samp|em|strong|q|ruby|bdo|youtube|references|ref|(?:dc.*?))(>| (?:.*?)>)/gmi
        to : (match) ->
          return match.replace('&lt;', '<')
      , # save the <pre> sections!
        rex : /<pre>((.|\n)*?)<\/pre>/gm
        to  : (match) ->
          self.placeholders.push match
          return '~~PLACEHOLDER' + (self.placeholders.length - 1) + '~~'
      , # HTML 5 animations
        rex : /(?:\[\[)(?:.*?:)(.+?).swf?(?:\]\])/g
        to  : (match, s1) -> return '<dcswiffy name="' + s1 + '"></dcswiffy>'
      , # Side Menu Language Links
        rex : /(\[\[.{2}:.+?\]\])/gmi
        to  : (match, s1) -> return ''
      , # link to tag or discipline search
        # [[tag:Some tag]] # [[discipline:Some discipline]]
        rex : /\[\[(Fachgebiet|Discipline|Tag):(.*?)\]\]/gi
        to  : (match) -> return ''
      , # internal link
        rex : /\[\[(?:\#([^\]]*?)\|)?(?:([^\]]*?)\|)?([^\]]*?)\]\]([a-zA-Z0-9äöüÄÖÜßéèêáàâóòôúùûíìîñÉÈÊÁÀÂÓÒÔÚÙÛÍÌÎ]*)/gi
        to  : (match, s1, s2, s3, s4) ->
          section = s1 || ''
          if s2? && s3?
            article = s2
            title   = s3 + s4
          else
            title = s3 + s4
            article = s3
          return '<a href="' + article + '" section="' + section + '" title="' + title + '">' + title + "</a>"
      , # internal link with title
        # [[An article to link to|With this custom title to be shown]]
        rex : /(?:\[\[)([^\]]+?)\|([^\]]+)(?:\]\])([a-zA-Z0-9]*)/gi
        to  : (match, s1, s2, s3) -> return '<a href="' + s1 + '" title="' + s2 + '">' + s2 + s3 + "</a>"
      , # internal link without title
        # [[An article to link to]]
        rex : /(?:\[\[)(.*?)(?:\]\])([a-zA-Z0-9]*)/gi
        to  : (match, s1, s2) -> return '<a href="' + s1 + '">' + s1 + s2 + "</a>"
      , # external link with title
        # [http://somekindoflink.com Some kind of title]
        rex : /(?:\[)(http:\/\/.+?) (.+?)\]/gi
        to  : (match, s1, s2) -> return '<a class="external-link" href="' + s1 + '" title="' + s2 + '">' + s2 + "</a>"
      , # external link without title
        # [http://somekindoflink.com]
        rex : /(?:\[)(http:\/\/.*?)(?:\])([a-zA-Z0-9]*)/gi
        to  : (match, s1, s2) -> return '<a class="external-link" href="' + s1 + '">' + s1 + s2 + "</a>"
      ]
      blocks : [
        # lists (order/unordered in any depth)
        rex : /((?:^|\n)(?:(?:[\*\#])(?:.|\n[\*\#])+)(?:[\n]|$))/gm,
        to  : (match) ->
          return self.parseCompleteList(match)
      , # wrap tables
        rex : /<table(.|\n)*?<\/table>/gm,
        to  : (match) -> return '<dc-table><ion-scroll has-bouncing="false" direction="x">' + match + '</ion-scroll></dc-table>'
      , # tables
        # {|
        # |
        #
        #rex : /\{\|((?:\n|.)*(?!\|\})(?:\n|.*))\|\}/gm
        rex : /\{\|(?:[^\n]*)((.|\n)*?)\|\}\n*/gm
        to : (match, tableinner) ->
          return self.parseTable(tableinner)
      , # p
        rex : /<p>((.|\n)*?)<\/p>/mgi
        to  : (match, s1) -> return s1
      ]
      inlines : [
        rex : /<br ?\/?>/gmi
        to  : (match) -> return '<br />'
      ,
        # h6
        # =====Headline 6=====
        rex : /(?:^|\n)\======(.*?)\======/mgi
        to  : (match, s1) -> return '\n<h6>'+s1+'</h6>\n'
      ,
        # h5
        # =====Headline 5=====
        rex : /(?:^|\n)\=====(.*?)\=====/mgi
        to  : (match, s1) -> return '\n<h5>'+s1+'</h5>\n'
      , # h4
        # ====Headline 4====
        rex : /(?:^|\n)\====(.*?)\====/mgi
        to  : (match, s1) -> return '\n<h4>'+s1+'</h4>\n'
      , # h3
        # ===Headline 3===
        rex : /(?:^|\n)\===(.*?)\===/mgi
        to  : (match, s1) -> return '\n<h3>'+s1+'</h3>\n'
      , # h2
        # ==Headline 2==
        rex : /(?:^|\n)\== *(.*?) *\==(?:[ \n]*)/mgi
        to  : (match, s1) -> return '\n<h2>'+s1+'</h2>\n'
      , # h1
        # =Headline 1=
        rex : /(?:^|\n)\= *(.*?) *\=(?:[ \n]*)/mgi
        to  : (match, s1) -> return '\n<h1>'+s1+'</h1>\n'
      , # p
        rex  : /(?:^|\n)(?!<)((?:.|\n[^\n<])+)/gmi
        #rex : /(?:^|\n)(?!<)(.+)/gi
        #rex : /(?:^|\n)(?![<=])((?:.|\n)*?)(?: *\n *\n)/gmi
        to  : (match, s1) ->
          return '<p>'+s1+'</p>'
      ,
        rex : /<p><\/p>/gmi
        to  : -> return ''
      ,
        # bold
        #'''text to be formatted bold'''
        rex : /'{3}(.*?)(?:'{3}|(\n|<br ?\/?>|$))/mgi
        to  : (match, s1, s2='') -> return '<strong>'+s1+'</strong>'+s2
      , # italic
        #''text to be formatted italic''
        rex : /'{2}(.*?)(?:'{2}|(\n|<br ?\/?>|$))/mgi
        to  : (match, s1, s2='') -> return '<em>'+s1+'</em>'+s2
      #, # standalone \n
      # rex : /(\n\n)/gi
      # to  : (match, s1) -> return '<br />'
      , # empty
        rex : /(?:^|\n)(?!<)( *)/gi
        to  : (match, s1) -> return ''
      , # standalone tags
        rex : /<(dc\w*) (.*?)\/>/g
        to  : (match, s1, s2) -> return '<' + s1 + ' ' + s2 + '></' + s1 + '>'
      ]

  ###*
   * [applyRules - apply all the rules defined above for either Wiki->HTML conversion or vice versa]
   * @param  {[String]} str      [String containing the Wiki/HTML content]
   * @param  {[String]} dir      [either 'toHTML' or 'toWiki']
   * @param  {[String]} language [language for getting info boxes from the flexikon directly]
   * @return {[String]}          [the parsed string]
  ###
  applyRules : (str, dir, language) ->
    for k, ruleblock of @rules[dir]
      for rule in ruleblock
        str = str.replace(rule.rex, rule.to)

    str = @replacePlaceholders(str, language)

    return str

  ###*
   * [toHTML - helper for calling applyRules accordingly; parses Wiki syntax to HTML]
   * @param  {[String]} str      [String containing the Wiki content]
   * @param  {[String]} language [language for getting info boxes from the flexikon directly]
   * @return {[String]}            [the parsed string]
  ###
  toHTML : (str, language) ->
    str = @applyRules(str, 'toHTML', language)
    #str.replace(/\n/g, '')
    return str

  ###*
   * [toWiki - helper for calling applyRules accordingly; parses HTML to Wiki syntax]
   * @param  {[String]} str      [String containing the Wiki content]
   * @param  {[String]} language [language for getting info boxes from the flexikon directly]
   * @return {[String]}            [the parsed string]
  ###
  toWiki : (str, language) ->
    return @applyRules(str, 'toWiki', language)

  ###*
   * [
   * replacePlaceholders - some parts must be replaced be placeholders
   * while parsing to ensure, they can be evaluated correctly afterwards
   * ]
   * @param  {[String]} str      [String containing the placeholders]
   * @param  {[String]} language [language for getting info boxes from the flexikon directly]
   * @return {[String]}            [the string with all contents]
  ###
  replacePlaceholders : (str, language) ->
    if !self.placeholders? || self.placeholders.length == 0
      return str

    for i in [(self.placeholders.length-1)..0]
      str = str.replace('~~PLACEHOLDER' + i + '~~', self.placeholders.pop().replace(/\{\{/g, '\\{\\{').replace(/"/g, '&quot;'))

    str = str.replace(/\<dc\-parse\-wiki/gm, '<dc-parse-wiki language="' + language + '"')

    return str

  ###*
   * DEPRECATED
   * @return
  ###
  stacktables : (expectedTableCount = 0, failCount = 0) ->
    if !GlobalSettings.Stacktables()
      return

    if expectedTableCount == 0
      return

    self_stacktables = @

    tables = angular.element('.article-html').last().find('table')

    if tables.length < expectedTableCount && failCount < 10
      $timeout(->
        self_stacktables.stacktables(expectedTableCount, failCount++)
      , 500)
      return

    for table in tables
      if !angular.element(table).hasClass('no-stack') && !angular.element(table).hasClass('stacktable') && angular.element(table).closest('dc-parse-wiki').length == 0
        angular.element(table).stacktable()
 
)