angular.module("Flexikon.controllers").controller('SearchCtrl', ($scope, $rootScope, $state, $controller, $filter, $timeout, $ionicScrollDelegate, FlexikonApi, Favorites, QuickSearch, Store, Network, DcDateTime) ->

  ###*
   * [track view in google analytics]
  ###
  $controller('TrackingCtrl', {$scope: $scope})
  
  $controller('BaseCtrl', {$scope: $scope})

  $scope.init = ->
    $scope.$on('$ionicView.enter', ->

      if arguments[1].direction != 'back'
        $scope.term.value = $scope.term.model = QuickSearch.getAndClear() || ''
        $scope.resetParams()
        $scope.search.items = []
        $scope.ui.loading = true
        $scope.loadMore()
        $ionicScrollDelegate.scrollTop()

      Favorites.getAll().then((result) ->
        $scope.search.favorites = []
        for fav in result
          $scope.search.favorites.push fav._id
      )

      $scope.isSummerbreak = undefined
      FlexikonApi.summerbreakCheck()
      .then((response) ->
        $scope.isSummerbreak = response.data['summer-break']

        $scope.summerbreak = {}
        $scope.summerbreak.popoverHint = ''

        if !isNaN(response.data['start-date']?)
          $scope.summerbreak.from = DcDateTime.getFormattedDate(new Date(response.data['start-date'] * 1000))

        if !isNaN(response.data['end-date']?)
          $scope.summerbreak.to = DcDateTime.getFormattedDate(new Date(response.data['end-date'] * 1000))

        if response.data['message']?.length > 0
          $scope.summerbreak.message = response.data['message']

        if $scope.summerbreak.from? && $scope.summerbreak.to?
          $scope.summerbreak.popoverHint += $filter(
            'translate'
            )(
              'DC_ARTICLE.SUMMERBREAK.POPOVER_HINT_DATE',
                from : $scope.summerbreak.from,
                to : $scope.summerbreak.to
            )
        else
          $scope.summerbreak.popoverHint += $filter(
            'translate'
          )(
            'DC_ARTICLE.SUMMERBREAK.POPOVER_HINT_PLAIN'
          )
      )
      .catch((error) ->
        if !$scope.isSummerbreak
          $scope.summerbreak =
            popoverHint : $filter(
              'translate'
            )(
              'DC_ARTICLE.SUMMERBREAK.POPOVER_HINT_FAILURE'
            )
            error       : true
      )
    )

    $scope.$on('$stateChangeStart', (event, toState, toParams, fromState, fromParams) ->
      if toState.name.match(/((.*)\.){2}/)[0] != fromState.name.match(/((.*)\.){2}/)[0]
        $scope.term.model = $scope.term.value = ''
        $scope.search.items = []
    )

    $scope.$watch("term.model", (n, o) ->
      $timeout.cancel($scope.suggestionTimeout)

      # no suggestions for offline search
      if $scope.ui.searchFavs
        return
      
      # no suggestions if current input equals last searched term
      if $scope.term.model == $scope.term.value
        $scope.resetSuggestionStyle()
        return

      # if term too short or equals search term
      if n.length <= 2 || n == $scope.term.value
        return

      #$scope.form.suggestions = []
      $scope.clearSuggestions = false

      $scope.suggestionTimeout = $timeout(->

        FlexikonApi.getSuggestions(n, 5).success((result) ->
          if $scope.clearSuggestions
            return

          $scope.form.newSuggestions = true
          $timeout(->
            $scope.form.newSuggestions = false
          , 1000)

          $scope.form.suggestions = result
          $scope.ui.suggestionStyle =
            'opacity'         : 1
            'webkit-transform': 'translateY(' + result.length * 36 + 'px)'
            'transform'       : 'translateY(' + result.length * 36 + 'px)'

        ).error((error) ->
          console.warn(error)
        )
      )
    )

  $scope.resume = ->
    $scope.listenToScroll = true
    $scope.onScroll()
    $scope.hideOnDrag()

  $scope.pause = ->
    $scope.listenToScroll = false

  $scope.$on("$ionicView.beforeEnter", ->
    $scope.$broadcast("search.beforeEnter", arguments)
    $scope.listenToScroll = true
    $scope.onScroll()
    $scope.hideOnDrag()

    document.removeEventListener('resume', $scope.resume)
    document.removeEventListener('pause', $scope.pause)

    document.addEventListener('resume', $scope.resume)
    document.addEventListener('pause', $scope.pause)
  )

  $scope.$on("$ionicView.leave", ->
    $scope.listenToScroll = false

    document.removeEventListener('resume', $scope.resume)
    document.removeEventListener('pause', $scope.pause)
  )

  $scope.scroll =
    interval   : 20
    currentStep: Date.now()
    start      : undefined

  $scope.term =
    model : ''
    value : ''
  
  $scope.ui =
    showSubheader     : true
    loading           : true
    suggestionStyle   :
      'webkit-transform': 'translateY(0)'
      'transform'       : 'translateY(0)'
    error             : false
    searchFavs        : false
    searchHint        : false
    onlineAgain       : false
    scrollPosition    : 0
    onTop             : true

  $scope.search =
    epp         : 20
    page        : 1
    items       : []
    inProgress  : false
    nomore      : false
    favorites   : []
    noExactMatch: false

  $scope.render =
    content : false

  $timeout ->
    $scope.render.content = true
  , 1000

  $scope.onScroll = ->
    searchScroll = $ionicScrollDelegate.$getByHandle('SearchScroll')
    if !searchScroll.getScrollPosition()? || Math.abs($scope.ui.scrollPosition - searchScroll.getScrollPosition().top) < 100
      return

    $scope.ui.scrollPosition = searchScroll.getScrollPosition().top
    if $scope.ui.onTop != ($scope.ui.scrollPosition < 1000)
      $timeout ->
        $scope.ui.onTop = $scope.ui.scrollPosition < 1000

    if $scope.listenToScroll
      ionic.DomUtil.requestAnimationFrame($scope.onScroll)

  $scope.hideOnDrag = ($event) ->
    $rootScope.$broadcast('hide-on-drag', {deltaY : $event.gesture.deltaY})

  $scope.hideOnDrag = ->
    if (Date.now() - $scope.scroll.currentStep) > $scope.scroll.interval
      searchScroll = $ionicScrollDelegate.$getByHandle('SearchScroll')
      delta = searchScroll.getScrollPosition().top - $scope.scroll.start
      $rootScope.$broadcast('hide-on-drag', {deltaY : delta * -1})

      $scope.scroll.currentStep = Date.now()

    if $scope.listenToScroll
      ionic.DomUtil.requestAnimationFrame($scope.hideOnDrag)

  $scope.hideOnDragRelease = ->
    $rootScope.$broadcast('hide-on-drag-release')

  $scope.hideOnDragTouch = ->
    searchScroll = $ionicScrollDelegate.$getByHandle('SearchScroll')
    $scope.scroll.start = searchScroll.getScrollPosition().top
    $scope.closeSuggestions()
    $rootScope.$broadcast('hide-on-drag-touch')

  $scope.onTouch = ->
    searchScroll = $ionicScrollDelegate.$getByHandle('SearchScroll')
    $scope.scroll.start = searchScroll.getScrollPosition().top
    $scope.closeSuggestions()
    $scope.hideOnDragTouch()

  $scope.resetParams = ->
    $scope.search.epp    = 20
    $scope.search.page   = 1
    $scope.search.nomore = false

  $scope.resetResults = ->
    $scope.resetParams()
    $scope.search.items = []

  $scope.resetResults()

  $scope.isFavorite = (title) ->
    return $scope.search.favorites.indexOf(title) >= 0

  $scope.form =
    initial           : true
    disciplineChanging: true
    infos             :
      amount            : 0
    discipline        : $filter('translate')('DC_ALL_DISCIPLINES')

  $scope.loadMore = (infinite) ->
    console.log('Loading more %s', infinite)


    if $scope.ui.onlineAgain
      $scope.ui.onlineAgain = false
      $scope.ui.searchFavs = false
      $scope.search.page = 1

    if $scope.search.page == 1
      $scope.search.noExactMatch = false

    if !$scope.search.inProgress
      $scope.ui.error          = false
      $scope.search.inProgress = true
      if $scope.search.items.length == 0 then $scope.ui.loading = true
      #FlexikonApi.getListing($scope.search.page, $scope.search.epp, $scope.form.keyword)
      FlexikonApi.getListing($scope.search.page, $scope.search.epp, $scope.term.value)
      .success((result) ->
        document.removeEventListener("online", $scope.loadMore, false)

        if result.length == 0 || !result.urls? then $scope.search.nomore = true
        $scope.search.items.push.apply($scope.search.items, result.urls)

        noExactMatch = false
        if $scope.search.page == 1 && $scope.search.items.length > 0 && $scope.term.value.length > 0
          noExactMatch = true
          for item in $scope.search.items
            if item.title == $scope.term.value
              noExactMatch = false
              break

        if $scope.search.page == 1 && $scope.search.items.length > 0 && $scope.term.value.length > 0
          FlexikonApi.getArticle($scope.term.value)
          .then((response) ->
            for k,v of response.data.query.pages
              article = v

            if article?.revisions?[0]?['*']? && (article.revisions[0]['*']).match(/^(#REDIRECT)|(#WEITERLEITUNG)/gmi)?.length > 0
              redirect = (article.revisions[0]['*']).match(/\[\[(.*)\]\]/)[1]

              i = 0
              found = false
              while i < $scope.search.items.length
                if $scope.search.items[i].title.toUpperCase() == redirect.toUpperCase()
                  $scope.search.items.unshift($scope.search.items.splice(i, 1)[0])
                  $scope.search.items[0].postfix = '(<span class="fa fa-arrow-right"> ' + $scope.term.value + '</span> )'
                  found = true
                  break
                i++

              $scope.search.noExactMatch = !found

            else
              $scope.search.noExactMatch = noExactMatch
          )
        else if $scope.search.page == 1
          $scope.search.noExactMatch = noExactMatch

        $scope.search.page++
        $scope.ui.loading        = false
        $scope.search.inProgress = false
        $ionicScrollDelegate.resize()
        $scope.$broadcast('scroll.infiniteScrollComplete')
      ).error((error) ->
        if !Store.isOwned('flexikon_offline')
          $scope.ui.error = true
        else
          $scope.ui.searchFavs = true
          $scope.ui.searchHint = true

        $scope.ui.loading        = false
        $scope.search.inProgress = false
        document.removeEventListener("online", $scope.onlineSearch, false)
        document.addEventListener("online", $scope.onlineSearch, false)
        $scope.$broadcast('scroll.infiniteScrollComplete')
      )

  $scope.onlineSearch = ->
    if $scope.ui.onlineAgain
      $scope.ui.onlineAgain = false
      $scope.searchFavs     = false
      $scope.loadMore()
      return

    if $scope.ui.searchFavs && $scope.search.favorites.length > 0
      $scope.ui.searchHint  = false
      $scope.ui.onlineAgain = true
      return

    $scope.loadMore()

  $scope.searchSubmit = ->
    $scope.form.suggestions = []
    $scope.clearSuggestions = true

    if $scope.ui.searchFavs && (!$scope.ui.onlineAgain && !Network.hasConnection())
      return
    else if $scope.ui.searchFavs
      $scope.ui.searchFavs = false
      $scope.ui.onlineAgain = true

    $ionicScrollDelegate.scrollTop()

    $scope.ui.onlineAgain = false
    $scope.ui.searchFavs = false
    $scope.ui.searchHint = false

    $scope.getArticlesForTerm()
    $scope.ui.loading = true

  $scope.searchClear = ->
    $scope.form.suggestions = []
    $scope.clearSuggestions = true

  $scope.getArticlesForTerm = ->
    $scope.form.suggestions = []
    $scope.resetResults()
    $scope.resetSuggestionStyle()
    $scope.loadMore()

  $scope.openArticle = (page, create=false, suggestion=false) ->
    if suggestion
      if analytics? && $scope.term then analytics.trackEvent(
        'Article'
        'autosuggest'
        page
      )

    page = page.replace(/&[^;]*;/g, '').replace(/<(.*?)>/g, '')
    $state.go('^.article', {page : window.decodeURIComponent(page), create : create})

    $timeout ->
      $scope.form.suggestions = []

  $scope.closeFavHint = ->
    $scope.ui.searchHint = false

  $scope.closeOnlineHint = ->
    $scope.ui.onlineAgain = false

  $scope.closeSuggestions = (event) ->
    if $scope.form.newSuggestions
      return

    $scope.resetSuggestionStyle()
    $timeout(->
      $scope.form.suggestions = []
      $scope.clearSuggestions = true
    )

  $scope.resetSuggestionStyle = ->
    $scope.ui.suggestionStyle =
      'opacity'         : 0
      'webkit-transform': 'translateY(0)'
      'transform'       : 'translateY(0)'

  $scope.init()

  return

)