# Controller
#

angular.module("Flexikon.controllers").controller('TabCtrl', (
  $scope
  $state
  $rootScope
  $timeout
  $ionicHistory
  $ionicScrollDelegate
  $ionicSideMenuDelegate
  $ionicTabsDelegate
  $controller
  Load
  Login
  TabHide
  NavBar
  Network
  Favorites
  FlexikonApi
  LastSeen
  Store
  ToC
  Theme
  GlobalSettings
  ExtendedState
  StateChangeInterrupter
  GuidedTour
  FullscreenViewer
) ->

  #Favorites.clear()
  #LastSeen.clear()

  ionic.Platform.ready(->

    $controller('ModalCtrl', {$scope: $scope})

    $ionicSideMenuDelegate.canDragContent(false)

    # Google Analytics initialization
    if ionic.Platform.isWebView()
      #analytics.debugMode()
      analytics.startTrackerWithId('UA-2421261-69')

    $timeout(->
      # Store initialization
      if store?
        Store.initialize()
        #store.verbosity = store.INFO
    
    , 1000)
  )

  $scope.STATES =
    'home'     : 0
    'favorites': 1
    'search'   : 2
    'settings' : 3
  
  $scope.tabs =
    loading : false

  $scope.login =
    required : !Login.isAuthenticated()

  $scope.connection =
    available : true

  $scope.rightButtons =
    type: 'button-clear'
    content: 'Left Button'
    tap: (e) ->
      $location.path("/")

  $scope.settings =
    theme: Theme.getSelected()
    stacktables: GlobalSettings.Stacktables() || false

  $scope.goTo = (state, index) ->
    sciCallback = StateChangeInterrupter.getCallback()
    if sciCallback?
      sciCallback('state', state, index)
      return

    $state.go(state, {}, {})
    #$ionicTabsDelegate.select(index)

  $scope.toggleTheme = ->
    $scope.settings.theme = Theme.toggle()

  $scope.toggleStacktables = ->
    $scope.settings.stacktables = !$scope.settings.stacktables
    GlobalSettings.Stacktables($scope.settings.stacktables)

  $scope.darkThemeSelected = ->
    return Theme.darkThemeSelected()

  $scope.navBarHidden = ->
    return false #NavBar.hidden()

  $scope.toggleSideMenu = ->
    $ionicSideMenuDelegate.toggleRight()

  $scope.tabsHidden = ->
    return TabHide.hidden()

  $scope.isLoading = ->
    return Load.isLoading()

  $scope.isBlockedByLoading = ->
    return Load.isBlocked()

  $scope.isLoginRequired = (required) ->
    if required == undefined
      return $scope.login.required || !Login.isAuthenticated()
    else
      $scope.login.required = required
      Load.isLoading(false)
      $scope.explApply()

      if required
        Login.deleteToken()

  $scope.hasConnection = ->
    return Network.hasConnection()
  
  $rootScope.$on 'event:auth-loginCancelled', ->
    $scope.isLoginRequired(true)

  $rootScope.$on 'event:auth-loginRequired', ->
    $scope.isLoginRequired(true)

  $rootScope.$on 'event:auth-loginConfirmed', ->
    $scope.isLoginRequired(false)
    #$ionicHistory.clearCache()
    #$ionicHistory.clearHistory()

  $rootScope.$on 'event:auth-logout', ->
    $scope.isLoginRequired(true)
    $ionicHistory.clearCache()
    $ionicHistory.clearHistory()
    

  ###
  #
  # This is necessary in addition to the ui-sref due to the fact that the AskForm has
  # to be able to interrupt the state change in case of unsaved changes.
  #
  # Even though the ui-sref notifies about the state change, its event's preventDefault
  # isn't actually preventing the change. Only an ng-click action might be prevented which
  # is triggered right here.
  #
  ###
  $scope.switchTab = (index) ->
    if index == 0
      $state.transitionTo('tabs.home.overview', {}, {notify: true})

    else if index == 1
      $state.transitionTo('tabs.favorites.overview', {}, {notify: true})

    else if index == 2
      $state.transitionTo('tabs.search.overview', {}, {notify: true})

    else if index == 3
      $state.transitionTo('tabs.settings.overview', {}, {notify: true})

  $scope.selectTabForState = (state) ->
    if state.indexOf('tabs.home') >= 0
      $ionicTabsDelegate.select(0)
    else if state.indexOf('tabs.favorites') >= 0
      $ionicTabsDelegate.select(1)
    else if state.indexOf('tabs.search') >= 0
      $ionicTabsDelegate.select(2)
    else if state.indexOf('tabs.settings') >= 0
      $ionicTabsDelegate.select(3)

  $scope.openStatic = (state) ->
    $scope.selectTabForState(state)
    $state.transitionTo(state, {}, {reload: true})
    $ionicSideMenuDelegate.toggleRight()

  $scope.openInSameBranch = (state) ->
    ExtendedState.openInSameBranch(state)
    if $ionicSideMenuDelegate.isOpenRight()
      $ionicSideMenuDelegate.toggleRight()

  $scope.explApply = ->
    if !$scope.$$phase
      $scope.$apply()

  $scope.isHome = ->
    return $state.current.name.indexOf('home') >= 0

  $scope.openUrl = (url) ->
    if cordova?
      ref = window.open(url, "_system")
    else
      window.open(url, "_blank")

  $scope.$on('$stateChangeStart', (event, toState, toParams, fromState, fromParams) ->
    Load.isLoading(false)

    newState = toState.name.match(/\.(.*?)\./)[1]
    if $ionicTabsDelegate.selectedIndex() != $scope.STATES[newState]
      $ionicTabsDelegate.select($scope.STATES[newState])

    if toState.name.match(/((.*)\.){2}/)[0] != fromState.name.match(/((.*)\.){2}/)[0]
      NavBar.unhide()
      $timeout(->
        if $state.current.name.indexOf('overview') > 0
          return

        $state.transitionTo(toState.name.match(/((.*)\.){2}/)[0] + 'overview', toParams,{reload: false, notify: true})
        $timeout(->
          NavBar.unhide()
        )
      )
  )

  $scope.runHomeOverlay = ->
    GuidedTour.setRootElement(undefined)
    GuidedTour.start(
      'FlexikonHomeIntro',
      GuidedTour.getSteps('homeSteps')
    )

  ionic.DomUtil.ready(->
    $timeout(->
      $scope.runHomeOverlay()
    , 500)
  )

)