angular.module("Flexikon.services").factory("FlexikonApi", ($http, $httpBackend, $timeout, $cacheFactory, $q, Api, FlexikonMocking, LocalStorage, GlobalSettings, Login) =>

  @baseUrl =
    production            : 'http://www.doccheck.com/'
    development           : 'http://apps.dev55.antwerpes.de/'
    development_mocked    : '/'

  @flexikonBaseUrl =
    production            : 'http://flexikon.doccheck.com/de/api.php'
    development           : 'http://doccheck-flexikon4.dev55.antwerpes.de/de/api.php'
    development_mocked    : '/api.php'

  @searchBaseUrl =
    production            : 'http://search.doccheck.com/autosuggest.php'
    development           : 'http://search.doccheck.com/autosuggest.php'
    development_mocked    : '/'

  @basePath      = 'api/'

  @api           =
    article     : 'flexikon_article/'
    top         : 'flexikon_top-article/'
    definition  : 'flexikon_definition'
    summerbreak : 'flexikon_summerbreak/'


  ###
  # ^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=
  # Utility
  # ^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=
  ###

  @getBaseUrl = =>
    return @baseUrl[Api.getEnv()]

  @getUrlForApi = (api) =>
    return @baseUrl[Api.getEnv()] + @basePath + @api[api]

  @getFlexikonUrlForLanguage = (language) =>
    if !language? then language = GlobalSettings.Language()
    url = @flexikonBaseUrl[Api.getEnv()].replace('/de/', '/' + language + '/')
    return url

  @getListing = (page=1, epp=18, term='') =>
    paramObject =
      page        : page
      epp         : epp
      dc_api_key  : Api.getApiKey()
      term        : term
      language    : GlobalSettings.Language()

    return $http({
      url: @getUrlForApi('article')
      method: "GET"
      cache: false
      params: paramObject
    })

  @getTopListing = (epp=3) =>
    paramObject =
      epp         : epp
      dc_api_key  : Api.getApiKey()
      language    : GlobalSettings.Language()

    return $http({
      url: @getUrlForApi('top')
      method: "GET"
      cache: false
      params: paramObject
    })

  @getDefinition= (page, language) =>
    if !language? then language = GlobalSettings.Language()

    paramObject =
      title       : page
      language    : language
      dc_api_key  : Api.getApiKey()

    return $http({
      url: @getUrlForApi('definition')
      method: "GET"
      cache: false
      params: paramObject
    })

  @getPreview = (page, section, language) =>
    if !language? then language = GlobalSettings.Language()

    paramObject =
      action     : 'parse'
      format     : 'json'
      page       : page
      section    : section
      disabletoc : true

    url = @getFlexikonUrlForLanguage(language)

    return $http({
      url: url
      method: "GET"
      cache: false
      params: paramObject
    })

  @getArticle = (page, language) =>
    if !language? then language = GlobalSettings.Language()

    paramObject =
      action : 'query'
      format : 'json'
      rvprop : 'content'
      prop   : 'revisions'
      titles : page

    url = @getFlexikonUrlForLanguage(language)

    ###
    return $http({
      url: url
      method: "GET"
      cache: false
      params: paramObject
    })
    ###

    return $http.get(url,
      params: paramObject
      cache: false
    )

  @summerbreakCheck = =>

    if !language? then language = GlobalSettings.Language()

    paramObject =
      action : 'dcissummerbreak'
      format : 'json'

    url = @getFlexikonUrlForLanguage(language)

    return $http.get(url,
      params: paramObject
      cache: false
      timeout: 10000
    )

  @getChannelListing = =>
    userId = LocalStorage.get('dc_flexikon_user_id') || 'no:user:specified'

    paramObject =
      action : 'query'
      list : 'usercontribs'
      ucuser : userId
      uclimit : 100
      ucdir : 'older'
      format : 'json'
      #ucshow : 'top'
      ucprop : 'title|timestamp'

    return $http({
      url: @getFlexikonUrlForLanguage(GlobalSettings.Language())
      method: "GET"
      cache: false
      params: paramObject
    })

  @getSuggestions = (q, count=5, mql=2) =>
    @suggestionCancel?.resolve()
    @suggestionCancel = $q.defer()

    paramObject =
      q           : q.toLowerCase()
      count       : count
      mql         : mql
      dictionary  : 'flexikon_title'

    return $http({
      url: @searchBaseUrl[Api.getEnv()]
      method: "GET"
      cache: false
      params: paramObject
      timeout: @suggestionCancel.promise
    })

  @getTemplate = (templatetext, language, successCallback, errorCallback, progressCallback) =>
    if !language? || language == 'undefined' 
      language = GlobalSettings.Language()

    fd = new FormData()

    fd.append 'text', templatetext

    xhr = new XMLHttpRequest()

    xhr.upload.addEventListener('progress', (e) ->
      if progressCallback
        progressCallback(e)
    , false)
    xhr.addEventListener("load", (e) ->
      if successCallback
        successCallback(JSON.parse(@response), e)
    , false)
    xhr.addEventListener("error", (e) ->
      if errorCallback
        errorCallback(e)
    , false)
    xhr.addEventListener("abort", (e) ->
      if errorCallback
        errorCallback(e)
    , false)

    xhr.open('POST', @getFlexikonUrlForLanguage(language) + '?action=parse&format=json&title=dummy')

    @setDocCheckHeader(xhr)

    xhr.setRequestHeader('Cache-Control', 'no-cache')
    xhr.setRequestHeader("Accept", "application/json")

    if startedCallback?
      startedCallback()
    xhr.send(fd)

  @post = (title, text, section, successCallback, errorCallback, progressCallback) =>

    fd = new FormData()

    fd.append 'title', title
    fd.append 'text', text
    fd.append 'language', GlobalSettings.Language()

    if section? && section != -1 then fd.append 'section', section

    xhr = new XMLHttpRequest()

    xhr.upload.addEventListener('progress', (e) ->
      if progressCallback
        progressCallback(e)
    , false)
    xhr.addEventListener("load", (e) ->
      if successCallback
        successCallback(@response, e)
    , false)
    xhr.addEventListener("error", (e) ->
      if errorCallback
        errorCallback(e)
    , false)
    xhr.addEventListener("abort", (e) ->
      if errorCallback
        errorCallback(e)
    , false)

    xhr.open('POST', @getUrlForApi('article') + '?dc_api_key=' + Api.getApiKey() + '&language=' + GlobalSettings.Language())

    xhr.setRequestHeader('Cache-Control', 'no-cache')
    xhr.setRequestHeader('Authorization', Login.getAccessToken())
    xhr.setRequestHeader('dc_api_key', Api.getApiKey())
    xhr.setRequestHeader("Accept", "application/json")

    @setDocCheckHeader(xhr)

    if startedCallback?
      startedCallback()
    xhr.send(fd)

  @setDocCheckHeader = (xhr) ->

    device = ionic.Platform.device()

    if !device?
      deviceInformation = []
    else
      deviceInformation = [
        'os=' + device.platform
        'os-version=' + device.version
        'manufacturer=' + device.manufacturer
        'device=' + device.model
      ]

    xhr.setRequestHeader('X-Doccheck-App', 'Flexikon App/1.4.0; ' + deviceInformation.join('; ') )

  return {
    getBaseUrl : @getBaseUrl
    getUrlForApi : @getUrlForApi
    getFlexikonUrlForLanguage : @getFlexikonUrlForLanguage
    setDocCheckHeader : @setDocCheckHeader

    getListing : @getListing
    getTopListing : @getTopListing
    getChannelListing : @getChannelListing

    getDefinition : @getDefinition
    getPreview : @getPreview
    getArticle : @getArticle

    getTemplate : @getTemplate
    getSuggestions : @getSuggestions

    summerbreakCheck : @summerbreakCheck

    post : @post
  }

)