angular.module("Flexikon.services").factory("PicturesApi", ($http, Api, GlobalSettings, Login) ->

  baseUrl   :
    production  : 'http://pictures.doccheck.com/'
    development : 'http://pictures-stage.dev55.antwerpes.de/'

  basePath  : ''

  pending   :
    listing     : undefined

  api       :
    photo       : 'api_photo/'
    photos      : 'api_photos/ids/'
    channel     : 'api_channel/'

  link      :
    photo       : 'photo/'

  ###
  # ^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=
  # Utility
  # ^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=
  ###

  getLinkFor: (type, language) ->
    if !language? then language = GlobalSettings.getLanguageForURL()
    return @baseUrl[Api.getEnv()] + @basePath + language + '/' + @link[type]

  getUrlForApi : (api) ->
    return @baseUrl[Api.getEnv()] + @basePath + @api[api]

  getPictures : (idArray) ->
    d = $.Deferred()

    pictures = []

    if idArray.length > 0
      @loadPictures(idArray, d)
    else
      d.resolve(pictures)

    return d

  loadPictures : (idArray, def, pictures = []) ->
    self = @

    paramObject =
      dc_api_key  : Api.getApiKey()

    $http({
      url : @getUrlForApi('photo') + idArray.shift()
      method : 'GET'
      params : paramObject
      cache : true
    }).success((response) ->
      pictures.push(response)

      if idArray.length > 0
        self.loadPictures(idArray, def, pictures)
      else
        def.resolve(pictures)
    ).error((error) ->
      console.warn(error)

      if idArray.length > 0
        self.loadPictures(idArray, def, pictures)
      else
        def.resolve(pictures)
    )

  getPicture : (id) ->
    paramObject =
      dc_api_key  : Api.getApiKey()

    return $http({
      url : @getUrlForApi('photo') + id
      method : 'GET'
      params : paramObject
      cache : true
    })

  getUserPictures: (page=1, epp=20) ->

    url = @getUrlForApi('channel') + 'token_owner'

    paramObject = {
      page: page
      epp: epp
      dc_api_key: Api.getApiKey()
      language: GlobalSettings.Language()
    }

    return $http({
      url: url,
      method: 'GET'
      cache: false
      timeout: 10000
      transformRequest: (obj) ->
        str = []
        for p of obj
          str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]))
        return str.join("&")
      params: paramObject
    })

  uploadPicture: (data, successCallback, errorCallback, startedCallback, progressCallback) ->

    createBlob = (base64str, mime) ->
      binary = atob(base64str.replace(/\s/g, ''))
      len = binary.length
      buffer = new ArrayBuffer(len)
      view = new Uint8Array(buffer)
      i = 0
      while i < len
        view[i] = binary.charCodeAt(i)
        i++
      blob = new Blob( [view], { type: mime })
      return blob

    imgData = data.substr(data.indexOf(',')+1)
    imgMime = data.substr(data.indexOf(':')+1, data.indexOf(';') - 1 - data.indexOf(':'))

    data = createBlob(imgData, imgMime)

    qUrl = @getUrlForApi('photo')
    qData = {
      Filedata: data
      language: GlobalSettings.Language()
    }

    createFileName = ->
      d = new Date()
      return ("" + d.getFullYear() + "-" + (d.getMonth()+1) + "-" + d.getDate() + "-" + d.getHours() + "-" + d.getMinutes() + "-" + d.getSeconds() + "-AskApp." + imgMime.substr(imgMime.indexOf('/')+1))

    fd = new FormData()
    fd.append('Filedata', data, createFileName())
    fd.append('language', GlobalSettings.Language())
    fd.append('visibility', 1)

    xhr = new XMLHttpRequest()

    xhr.upload.addEventListener('progress', (e) ->
      if progressCallback
        progressCallback(e)
    , false)
    xhr.addEventListener("load", (e) ->
      if successCallback
        successCallback(@response, e)
    , false)
    xhr.addEventListener("error", (e) ->
      if errorCallback
        errorCallback(e)
    , false)
    xhr.addEventListener("abort", (e) ->
      if errorCallback
        errorCallback(e)
    , false)

    xhr.open('POST', qUrl)

    xhr.setRequestHeader('Cache-Control', 'no-cache')
    xhr.setRequestHeader('Authorization', Login.getAccessToken())
    xhr.setRequestHeader('dc_api_key', Api.getApiKey())
    xhr.setRequestHeader("Accept", "application/json")

    if startedCallback
      startedCallback()
    xhr.send(fd)

  getPicturesForMediaViewer : (ids) ->
    paramObject =
      'ids[]' : ids
      dc_api_key : Api.getApiKey()

    return $http({
      url: @getUrlForApi('photos')
      method: "GET"
      cache: false
      params: paramObject
    })


)